""" Code from `colander` package """
import re
from interfaces import _, Invalid


class All(object):
    """ Composite validator which succeeds if none of its
    subvalidators raises an :class:`Invalid` exception"""

    def __init__(self, *validators):
        self.validators = list(validators)

    def __call__(self, field, value):
        msgs = []
        for validator in self.validators:
            try:
                validator(field, value)
            except Invalid, e:
                msgs.append(e.msg)

        if msgs:
            raise Invalid(field, msgs)


class Function(object):
    """ Validator which accepts a function and an optional message;
    the function is called with the ``value`` during validation.

    If the function returns anything falsy (``None``, ``False``, the
    empty string, ``0``, an object with a ``__nonzero__`` that returns
    ``False``, etc) when called during validation, an
    :exc:`ptah.form.Invalid` exception is raised (validation fails);
    its msg will be the value of the ``message`` argument passed to
    this class' constructor.

    If the function returns a stringlike object (a ``str`` or
    ``unicode`` object) that is *not* the empty string , a
    :exc:`ptah.form.Invalid` exception is raised using the stringlike
    value returned from the function as the exeption message
    (validation fails).

    If the function returns anything *except* a stringlike object
    object which is truthy (e.g. ``True``, the integer ``1``, an
    object with a ``__nonzero__`` that returns ``True``, etc), an
    :exc:`ptah.form.Invalid` exception is *not* raised (validation
    succeeds).

    The default value for the ``message`` when not provided via the
    constructor is ``Invalid value``.
    """

    def __init__(self, function, message=_('Invalid value')):
        self.function = function
        self.message = message

    def __call__(self, field, value):
        result = self.function(value)

        if not result:
            raise Invalid(field, self.message)

        if isinstance(result, basestring):
            raise Invalid(field, result)


class Regex(object):
    """ Regular expression validator.

        Initialize it with the string regular expression ``regex``
        that will be compiled and matched against ``value`` when
        validator is called. If ``msg`` is supplied, it will be the
        error message to be used; otherwise, defaults to 'String does
        not match expected pattern'.

        The ``regex`` argument may also be a pattern object (the
        result of ``re.compile``) instead of a string.

        When calling, if ``value`` matches the regular expression,
        validation succeeds; otherwise, :exc:`ptah.form.Invalid` is
        raised with the ``msg`` error message.
    """

    def __init__(self, regex, msg=None):
        if isinstance(regex, basestring):
            self.match_object = re.compile(regex)
        else:
            self.match_object = regex
        if msg is None:
            self.msg = _("String does not match expected pattern")
        else:
            self.msg = msg

    def __call__(self, field, value):
        if self.match_object.match(value) is None:
            raise Invalid(field, self.msg)


class Email(Regex):
    """ Email address validator. If ``msg`` is supplied, it will be
        the error message to be used when raising :exc:`ptah.form.Invalid`;
        otherwise, defaults to 'Invalid email address'.
    """

    def __init__(self, msg=None):
        if msg is None:
            msg = _("Invalid email address")

        super(Email, self).__init__(
            u'(?i)^[A-Z0-9._%+-]+@[A-Z0-9.-]+\.[A-Z]{2,4}$', msg=msg)


class Range(object):
    """ Validator which succeeds if the value it is passed is greater
    or equal to ``min`` and less than or equal to ``max``.  If ``min``
    is not specified, or is specified as ``None``, no lower bound
    exists.  If ``max`` is not specified, or is specified as ``None``,
    no upper bound exists.

    ``min_err`` is used to form the ``msg`` of the
    :exc:`ptah.form.Invalid` error when reporting a validation failure
    caused by a value not meeting the minimum.  If ``min_err`` is
    specified, it must be a string.  The string may contain the
    replacement targets ``${min}`` and ``${val}``, representing the
    minimum value and the provided value respectively.  If it is not
    provided, it defaults to ``'${val} is less than minimum value
    ${min}'``.

    ``max_err`` is used to form the ``msg`` of the
    :exc:`ptah.form.Invalid` error when reporting a validation failure
    caused by a value exceeding the maximum.  If ``max_err`` is
    specified, it must be a string.  The string may contain the
    replacement targets ``${max}`` and ``${val}``, representing the
    maximum value and the provided value respectively.  If it is not
    provided, it defaults to ``'${val} is greater than maximum value
    ${max}'``.
    """
    min_err = _('${val} is less than minimum value ${min}')
    max_err = _('${val} is greater than maximum value ${max}')

    def __init__(self, min=None, max=None, min_err=None, max_err=None):
        self.min = min
        self.max = max
        if min_err is not None:
            self.min_err = min_err
        if max_err is not None:
            self.max_err = max_err

    def __call__(self, field, value):
        if self.min is not None:
            if value < self.min:
                min_err = _(self.min_err,
                            mapping={'val': value, 'min': self.min})
                raise Invalid(field, min_err)

        if self.max is not None:
            if value > self.max:
                max_err = _(self.max_err,
                            mapping={'val': value, 'max': self.max})
                raise Invalid(field, max_err)


class Length(object):
    """ Validator which succeeds if the value passed to it has a
    length between a minimum and maximum.  The value is most often a
    string."""

    def __init__(self, min=None, max=None):
        self.min = min
        self.max = max

    def __call__(self, field, value):
        if self.min is not None:
            if len(value) < self.min:
                min_err = _('Shorter than minimum length ${min}',
                            mapping={'min': self.min})
                raise Invalid(field, min_err)

        if self.max is not None:
            if len(value) > self.max:
                max_err = _('Longer than maximum length ${max}',
                            mapping={'max': self.max})
                raise Invalid(field, max_err)


class OneOf(object):
    """ Validator which succeeds if the value passed to it is one of
    a fixed set of values """

    def __init__(self, choices):
        self.choices = choices

    def __call__(self, field, value):
        if not value in self.choices:
            choices = ', '.join(['%s' % x for x in self.choices])
            err = _('"${val}" is not one of ${choices}',
                    mapping={'val': value, 'choices': choices})
            raise Invalid(field, err)



""" content types module """
import ptah
from ptah import cms, view, form, manage

from pyramid.decorator import reify
from pyramid.httpexceptions import HTTPFound

class ModelModule(manage.PtahModule):
    __doc__ = u'A listing of all registered models.'

    title = 'Models'
    manage.module('models')

    def __getitem__(self, key):
        ti = cms.get_type('cms-type:%s'%key)

        if ti is not None:
            return Model(ti, self, self.request)

        raise KeyError(key)


class Model(object):

    def __init__(self, tinfo, context, request):
        self.__name__ = tinfo.name
        self.__parent__ = context
        self.request = request
        self.tinfo = tinfo

    def __getitem__(self, key):
        if key == 'add.html':
            raise KeyError(key)

        try:
            return Record(key, self.tinfo, self, self.request)
        except:
            raise KeyError(key)


class Record(object):

    def __init__(self, pid, tinfo, parent, request):
        self.__name__ = str(pid)
        self.__parent__ = parent

        self.pid = pid
        self.tinfo = tinfo
        self.cls = tinfo.cls
        self.request = request

        self.record = cms.Session.query(tinfo.cls)\
            .filter(tinfo.cls.__id__ == pid).one()


class ModelModuleView(view.View):
    view.pview(context = ModelModule,
               template = view.template('ptah.manage:templates/models.pt'))

    rst_to_html = staticmethod(ptah.rst_to_html)

    def update(self):
        types = []
        for ti in cms.get_types().values():
            if ti.__uri__ in manage.CONFIG['disable_models']:
                continue
            types.append((ti.title, ti))
        types.sort()
        self.types = [f for _t, f in types]


class ModelView(form.Form):
    view.pview(context = Model,
               template = view.template('ptah.manage:templates/model.pt'))

    csrf = True
    page = ptah.Pagination(15)

    def update(self):
        tinfo = self.context.tinfo
        cls = tinfo.cls

        self.fields = tinfo.fieldset

        super(ModelView, self).update()

        request = self.request
        try:
            current = int(request.params.get('batch', None))
            if not current:
                current = 1

            request.session['table-current-batch'] = current
        except:
            current = request.session.get('table-current-batch')
            if not current:
                current = 1

        self.size = cms.Session.query(cls).count()
        self.current = current

        self.pages, self.prev, self.next = self.page(self.size, self.current)

        offset, limit = self.page.offset(current)
        self.data = cms.Session.query(cls)\
            .order_by(cls.__id__).offset(offset).limit(limit).all()

    def get_record_info(self, item):
        res = {}
        for field in self.widgets.fields():
            val = getattr(item, field.name, field.default)
            res[field.name] = field.serialize(val)

        return res

    @form.button('Add', actype=form.AC_PRIMARY)
    def add(self):
        raise HTTPFound(location='add.html')

    @form.button('Remove', actype=form.AC_DANGER)
    def remove(self):
        self.validate_csrf_token()

        ids = []
        for id in self.request.POST.getall('rowid'):
            try:
                ids.append(int(id))
            except: # pragma: no cover
                pass

        if not ids:
            self.message('Please select records for removing.', 'warning')
            return

        for rec in cms.Session.query(self.context.tinfo.cls).filter(
            self.context.tinfo.cls.__id__.in_(ids)).all():
            cms.Session.delete(rec)

        self.message('Select records have been removed.')
        raise HTTPFound(location = self.request.url)


class AddRecord(form.Form):
    view.pview('add.html',
               context = Model,
               template = view.template('ptah.manage:templates/model-add.pt'))

    __doc__ = "Add model record."

    csrf = True

    @reify
    def fields(self):
        return self.context.tinfo.fieldset

    @form.button('Add', actype=form.AC_PRIMARY)
    def add_handler(self):
        data, errors = self.extract()

        if errors:
            self.message(errors, 'form-error')
            return

        self.record = record = self.context.tinfo.create()

        if hasattr(record, 'update'):
            record.update(**data)
        else:
            for field in self.context.tinfo.fieldset.fields():
                val = data.get(field.name, field.default)
                if val is not form.null:
                    setattr(record, field.name, val)

        cms.Session.add(record)
        cms.Session.flush()

        self.message('New record has been created.', 'success')
        raise HTTPFound(location='./%s/'%record.__id__)

    @form.button('Back')
    def back_handler(self):
        raise HTTPFound(location='.')


class EditRecord(form.Form):
    view.pview(context = Record,
               template = view.template('ptah.manage:templates/model-edit.pt'))

    __doc__ = "Edit model record."

    csrf = True

    @reify
    def label(self):
        return 'Record id: %s'%self.context.__name__

    @reify
    def fields(self):
        return self.context.tinfo.fieldset

    def update(self):
        self.manage_url = manage.CONFIG.manage_url
        super(EditRecord, self).update()

    def form_content(self):
        data = {}
        for field in self.fields.fields():
            data[field.name] = getattr(
                self.context.record, field.name, field.default)
        return data

    @form.button('Modify', actype=form.AC_PRIMARY)
    def modify_handler(self):
        data, errors = self.extract()

        if errors:
            self.message(errors, 'form-error')
            return

        record = self.context.record

        if hasattr(record, 'update'):
            record.update(**data)
        else:
            for field in self.fields.fields():
                val = data.get(field.name, field.default)
                if val is not form.null:
                    setattr(record, field.name, val)

        self.message('Model record has been modified.', 'success')

    @form.button('Back')
    def back_handler(self):
        raise HTTPFound(location='../')


class TypeIntrospection(object):
    """ Ptah content types """

    name = 'ptah-cms:type'
    title = 'Content Types'
    manage.introspection('ptah-cms:type')

    actions = view.template('ptah.manage:templates/directive-type.pt')

    def __init__(self, request):
        self.request = request

    def renderActions(self, *actions):
        return self.actions(
            types = cms.get_types(),
            actions = actions,
            rst_to_html = ptah.rst_to_html,
            manage_url = manage.CONFIG.manage_url,
            request = self.request)



""" password tool """
import colander
import translationstring
from os import urandom
from codecs import getencoder
from hashlib import sha1
from base64 import urlsafe_b64encode, urlsafe_b64decode
from datetime import timedelta

import ptah
from ptah import config, form, token

_ = translationstring.TranslationStringFactory('ptah')


PWD_CONFIG = config.register_settings(
    'ptah-password',

    config.SchemaNode(
        colander.Str(),
        name = 'manager',
        title = 'Password manager',
        description = 'Available password managers '\
            '("plain", "ssha", "bcrypt")',
        default = 'plain'),

    config.SchemaNode(
        colander.Int(),
        name = 'min_length',
        title = 'Length',
        description = 'Password minimium length.',
        default = 5),

    config.SchemaNode(
        colander.Bool(),
        name = 'letters_digits',
        title = 'Letters and digits',
        description = 'Use letters and digits in password.',
        default = False),

    config.SchemaNode(
        colander.Bool(),
        name = 'letters_mixed_case',
        title = 'Letters mixed case',
        description = 'Use letters in mixed case.',
        default = False),

    title = 'Password tool settings',
    )


PASSWORD_CHANGER_ID = 'ptah.password:changer'


TOKEN_TYPE = token.TokenType(
    '35c9b7df958f4e93ae9b275a7dc2219e', timedelta(minutes=10))


class PlainPasswordManager(object):
    """PLAIN password manager."""

    def encode(self, password, salt=None):
        return '{plain}%s' % password

    def check(self, encoded, password):
        if encoded != password:
            return encoded == '{plain}%s' % password
        return True


class SSHAPasswordManager(object):
    """SSHA password manager."""

    _encoder = getencoder("utf-8")

    def encode(self, password, salt=None):
        if salt is None:
            salt = urandom(4)
        hash = sha1(self._encoder(password)[0])
        hash.update(salt)
        return '{ssha}' + urlsafe_b64encode(hash.digest() + salt)

    def check(self, encoded_password, password):
        # urlsafe_b64decode() cannot handle unicode input string. We
        # encode to ascii. This is safe as the encoded_password string
        # should not contain non-ascii characters anyway.
        encoded_password = encoded_password.encode('ascii')
        byte_string = urlsafe_b64decode(encoded_password[6:])
        salt = byte_string[20:]
        return encoded_password == self.encode(password, salt)


class PasswordTool(object):
    """ Password management utility. """

    pm = {'{plain}': PlainPasswordManager(),
          '{ssha}': SSHAPasswordManager(),
          }

    @property
    def manager(self):
        try:
            return self.pm['{%s}' % PWD_CONFIG.manager]
        except KeyError:
            return self.pm['{plain}']

    def check(self, encoded, password):
        """ check encoded password with plain password """
        try:
            pm, pwd = encoded.split('}', 1)
        except:
            return self.manager.check(encoded, password)

        manager = self.pm.get('%s}' % pm)
        if manager is not None:
            return manager.check(encoded, password)
        return False

    def encode(self, password, salt=None):
        """ encode password with current password manager """
        return self.manager.encode(password, salt)

    def can_change_password(self, principal):
        """ can principal password be changed """
        return ptah.extract_uri_schema(principal.uri) in \
            config.get_cfg_storage(PASSWORD_CHANGER_ID)

    def get_principal(self, passcode):
        """ generate passcode for principal """
        data = token.service.get(passcode)

        if data is not None:
            return ptah.resolve(data)

    def generate_passcode(self, principal):
        """ generate passcode for principal """
        return token.service.generate(TOKEN_TYPE, principal.uri)

    def remove_passcode(self, passcode):
        """ remove passcode """
        token.service.remove(passcode)

    def change_password(self, passcode, password):
        """ change password """
        principal = self.get_principal(passcode)

        self.remove_passcode(passcode)

        if principal is not None:
            changers = config.get_cfg_storage(PASSWORD_CHANGER_ID)

            changer = changers.get(ptah.extract_uri_schema(principal.uri))
            if changer is not None:
                changer(principal, self.encode(password))
                return True

        return False

    def validate(self, password):
        """ Validate password """
        if len(password) < PWD_CONFIG.min_length:
            #return _('Password should be at least ${count} characters.',
            #         mapping={'count': self.min_length})
            return 'Password should be at least %s characters.' % \
                PWD_CONFIG.min_length
        elif PWD_CONFIG.letters_digits and \
                (password.isalpha() or password.isdigit()):
            return _('Password should contain both letters and digits.')
        elif PWD_CONFIG.letters_mixed_case and \
                (password.isupper() or password.islower()):
            return _('Password should contain letters in mixed case.')


pwd_tool = PasswordTool()


def password_changer(schema):
    info = config.DirectiveInfo()

    def wrapper(changer):
        info.attach(
            config.Action(
                lambda config, schema, changer: \
                    config.get_cfg_storage(PASSWORD_CHANGER_ID).update(
                            {schema: changer}),
                (schema, changer),
                discriminator=(PASSWORD_CHANGER_ID, schema))
            )

        return changer

    return wrapper


def passwordValidator(field, appstruct):
    err = pwd_tool.validate(appstruct)
    if err is not None:
        raise form.Invalid(field, err)


def passwordSchemaValidator(field, appstruct):
    if appstruct['password'] and appstruct['confirm_password']:
        if appstruct['password'] != appstruct['confirm_password']:
            raise form.Invalid(
                field, _("Password and Confirm Password should be the same."))

        passwordValidator(field, appstruct['password'])


PasswordSchema = form.Fieldset(

    form.FieldFactory(
        'password',
        'password',
        title = _(u'Password'),
        description = _(u'Enter password. '\
                        u'No spaces or special characters, should contain '\
                        u'digits and letters in mixed case.'),
        default = u''),

    form.FieldFactory(
        'password',
        'confirm_password',
        title = _(u'Confirm password'),
        description = _(u'Re-enter the password. '
                        u'Make sure the passwords are identical.'),
        default = u''),

    validator = passwordSchemaValidator
)



import ptah
from ptah import cms, config
from pyramid.testing import DummyRequest
from pyramid.httpexceptions import HTTPFound

from base import Base

class TestApp1(cms.ApplicationRoot):
    __type__ = cms.Type('app1')


class TestApp2(cms.ApplicationRoot):
    __type__ = cms.Type('app2')


class TestAppsModule(Base):

    def tearDown(self):
        config.cleanup_system(self.__class__.__module__)
        super(TestAppsModule, self).tearDown()

    def test_apps_module(self):
        from ptah.manage.manage import CONFIG, PtahManageRoute
        from ptah.manage.apps import ApplicationsModule

        request = DummyRequest()

        ptah.authService.set_userid('test')
        CONFIG['managers'] = ('*',)
        mr = PtahManageRoute(request)
        mod = mr['apps']

        self.assertIsInstance(mod, ApplicationsModule)

    def test_apps_view(self):
        from ptah.manage.apps import ApplicationsModule
        from ptah.manage.apps import ApplicationsModuleView

        cms.ApplicationFactory(
            TestApp1, '/test1', 'app1', 'Root App 1')

        cms.ApplicationFactory(
            TestApp2, '/test2', 'app2', 'Root App 2')

        self._init_ptah()

        request = DummyRequest()

        mod = ApplicationsModule(None, request)

        res = ApplicationsModuleView.__renderer__(mod, request)
        self.assertEqual(res.status, '200 OK')
        self.assertIn('/test1/', res.body)
        self.assertIn('/test2/', res.body)
        self.assertIn('App1', res.body)
        self.assertIn('App2', res.body)

    def test_apps_traverse(self):
        from ptah.manage.apps import ApplicationsModule
        from ptah.manage.apps import AppFactory

        factory1 = cms.ApplicationFactory(
            TestApp1, '/test1', 'app1', 'Root App 1')

        factory2 = cms.ApplicationFactory(
            TestApp2, '/test2', 'app2', 'Root App 2')

        self._init_ptah()

        request = DummyRequest()

        mod = ApplicationsModule(None, request)

        item = mod['app1']
        self.assertIsInstance(item, AppFactory)
        self.assertIs(item.factory, factory1)
        self.assertIsInstance(item.app, TestApp1)

        item = mod['app2']
        self.assertIsInstance(item, AppFactory)
        self.assertIs(item.factory, factory2)
        self.assertIsInstance(item.app, TestApp2)
        self.assertRaises(KeyError, mod.__getitem__, 'app3')


class TestAppSharingForm(Base):

    def tearDown(self):
        config.cleanup_system(self.__class__.__module__)
        super(TestAppSharingForm, self).tearDown()

    def _make_app(self, request=None):
        from ptah.manage.manage import CONFIG, PtahManageRoute

        class Principal(object):
            id = 'test-user'
            uri = 'test:user'
            login = 'admin@ptahproject.org'

        principal = Principal()

        @ptah.resolver('test')
        def principalResolver(uri):
            return principal

        @ptah.principal_searcher('test')
        def principalSearcher(term):
            return (principal,)

        cms.ApplicationFactory(
            TestApp1, '/test1', 'app1', 'Root App 1')

        cms.ApplicationFactory(
            TestApp2, '/test2', 'app2', 'Root App 2')

        self.TestRole = ptah.Role('test', 'Test role')

        self._init_ptah()

        if request is None:
            request = DummyRequest()

        ptah.authService.set_userid(ptah.SUPERUSER_URI)
        CONFIG['managers'] = ('*',)
        mr = PtahManageRoute(request)
        mod = mr['apps']
        return mod['app1']

    def test_sharingform_search(self):
        from ptah.manage.apps import SharingForm

        app = self._make_app()

        form = SharingForm(app, DummyRequest(
                POST={'form.buttons.search': 'Search',
                      'term': 'search term'}))
        form.csrf = False
        try:
            form.update()
        except Exception, res:
            pass

        self.assertIsInstance(res, HTTPFound)
        self.assertIn('apps-sharing-term', form.request.session)
        self.assertEqual(
            form.request.session['apps-sharing-term'], 'search term')

    def test_sharingform_search_error(self):
        from ptah.manage.apps import SharingForm

        app = self._make_app()

        form = SharingForm(app, DummyRequest(
                POST={'form.buttons.search': 'Search'}))
        form.csrf = False
        form.update()

        self.assertIn('Please specify search term',
                      form.request.session['msgservice'][0])

    def test_sharingform_update(self):
        from ptah.manage.apps import SharingForm

        app = self._make_app()

        form = SharingForm(app, DummyRequest(
                session={'apps-sharing-term': 'email'}))
        form.csrf = False
        form.update()

        self.assertEqual(len(form.users), 1)
        self.assertEqual(form.users[0].uri, 'test:user')
        self.assertIs(form.local_roles, app.app.__local_roles__)
        self.assertEqual(len(form.roles), 1)
        self.assertIs(form.roles[0], self.TestRole)
        self.assertEqual(form.get_principal('test:user').uri, 'test:user')

    def test_sharingform_save(self):
        from ptah.manage.apps import SharingForm

        app = self._make_app()

        form = SharingForm(app, DummyRequest(
                session={'apps-sharing-term': 'email'},
                POST={'form.buttons.save': 'Save',
                      'userid-test:user': 'on',
                      'user-test:user-role:test': 'on'}))
        form.csrf = False
        form.update()

        self.assertIn('test:user', app.app.__local_roles__)
        self.assertIn(self.TestRole.id, app.app.__local_roles__['test:user'])

    def test_sharingform_unset(self):
        from ptah.manage.apps import SharingForm

        app = self._make_app()
        app.app.__local_roles__ = {'test:user': [self.TestRole.id]}

        form = SharingForm(app, DummyRequest(
                session={'apps-sharing-term': 'email'},
                POST={'form.buttons.save': 'Save',
                      'userid-test:user': 'on'}))
        form.csrf = False
        form.update()

        self.assertNotIn('test:user', app.app.__local_roles__)



"""Form implementation"""
from collections import OrderedDict
from pyramid import security
from pyramid.decorator import reify
from pyramid.httpexceptions import HTTPForbidden
from webob.multidict import MultiDict

from ptah import view, config
from ptah.form.field import Field, Fieldset
from ptah.form.button import Buttons, Actions
from ptah.form.interfaces import _, Invalid, FORM_INPUT, FORM_DISPLAY
from ptah.form.interfaces import IForm

CSRF = None


def setCsrfUtility(util):
    global CSRF
    CSRF = util


class FormErrorMessage(view.Message):
    config.adapter(None, name='form-error')

    formErrorsMessage = _(u'Please fix indicated errors.')

    template = view.template('ptah.form:templates/form-error.pt')

    def render(self, message):
        errors = [err for err in message
                  if isinstance(err, Invalid) and err.field is None]

        return self.template(
            errors=errors,
            message=self.formErrorsMessage,
            request=self.request)


class FormWidgets(OrderedDict):

    mode = FORM_INPUT
    prefix = 'widgets.'
    fieldsets = ()

    def __init__(self, fields, form, request):
        self.form_fields = fields
        self.form = form
        self.request = request

        super(FormWidgets, self).__init__()

    def fields(self):
        return self.fieldset.fields()

    def update(self):
        params = self.form.form_params()
        content = self.form.form_content()
        prefix = '%s%s' % (self.form.prefix, self.prefix)

        self.fieldset = self.form_fields.bind(content, params)
        self.fieldsets = fieldsets = []

        # Walk through each field, making a widget out of it.
        for fieldset in self.fieldset.fieldsets():
            widgets = []

            for widget in fieldset.fields():
                if widget.mode is None:
                    widget.mode = self.mode
                widget.id = ('%s%s' % (prefix, widget.name)).replace('.', '-')
                widget.update(self.request)
                widgets.append(widget)
                self[widget.name] = widget

            fieldsets.append(
                {'fieldset': fieldset,
                 'name': fieldset.name,
                 'legend': fieldset.legend,
                 'widgets': widgets})

    def extract(self):
        data, errors = self.fieldset.extract()

        # additional form validation
        self.form.validate(data, errors)

        # set errors to fields
        for err in errors:
            if isinstance(err.field, Field) and err.field.error is None:
                err.field.error = err

        return data, errors


class Form(view.View):
    """A form

    """

    fields = Fieldset()
    buttons = None

    #: Form label
    label = None

    #: Form description
    description = ''

    #: Form prefix, it used for html elements `id` generations.
    prefix = 'form.'

    #: Instance of py:class:`ptah.form.Actions` class
    actions = None

    #: Instance of py:class:`FormWidgets` class
    widgets = None

    #: Form content, it should be `None` or dictionary with data for fields.
    content = None

    #: Form mode. It can be py:data::`ptah.form.FORM_INPUT` or
    #: py:data::`ptah.form.FORM_DISPLAY`
    mode = FORM_INPUT

    method = 'post'
    enctype = 'multipart/form-data'
    accept = None
    acceptCharset = None

    csrf = False
    csrfname = 'csrf-token'

    params = MultiDict({})

    def __init__(self, context, request):
        super(Form, self).__init__(context, request)

        if self.buttons is None:
            self.buttons = Buttons()

    @reify
    def action(self):
        return self.request.url

    @reify
    def name(self):
        return self.prefix.strip('.')

    @reify
    def id(self):
        return self.name.replace('.', '-')

    def form_content(self):
        return self.content

    def form_params(self):
        if self.method == 'post':
            return self.request.POST
        elif self.method == 'get':
            return self.request.GET
        elif self.method == 'params':
            return self.params
        else:
            return self.params

    def update_widgets(self):
        self.widgets = FormWidgets(self.fields, self, self.request)
        self.widgets.mode = self.mode
        self.widgets.update()

    def update_actions(self):
        self.actions = Actions(self, self.request)
        self.actions.update()

    @property
    def token(self):
        if CSRF is not None:
            return CSRF.generate(self.tokenData)

    @reify
    def tokenData(self):
        return '%s.%s:%s' % (self.__module__, self.__class__.__name__,
                           security.authenticated_userid(self.request))

    def validate(self, data, errors):
        self.validate_csrf_token()

    def validate_csrf_token(self):
        # check csrf token
        if self.csrf:
            token = self.form_params().get(self.csrfname, None)
            if token is not None:
                if CSRF is not None:
                    if CSRF.get(token) == self.tokenData:
                        return

            raise HTTPForbidden("Form authenticator is not found.")

    def extract(self):
        return self.widgets.extract()

    def update(self, **data):
        if not self.content and data:
            self.content = data

        self.update_widgets()
        self.update_actions()

        return self.actions.execute()

    def render(self):
        if self.template is None:
            return self.snippet(FORM_VIEW, self)

        kwargs = {'view': self,
                  'context': self.context,
                  'request': self.request}

        return self.template(**kwargs)


class DisplayForm(Form):

    mode = FORM_DISPLAY

    def form_params(self):
        return self.params


FORM_VIEW = 'form-view'
FORM_ACTIONS = 'form-actions'
FORM_WIDGET = 'form-widget'
FORM_DISPLAY_WIDGET = 'form-display-widget'

view.snippettype(FORM_VIEW, IForm, 'Form view')
view.snippettype(FORM_ACTIONS, IForm, 'Form actions')
view.snippettype(FORM_WIDGET, Field, 'Form widget')
view.snippettype(FORM_DISPLAY_WIDGET, Field, 'Form display widget')

view.register_snippet(
    'form-view', Form,
    template=view.template('ptah.form:templates/form.pt'))

view.register_snippet(
    'form-view', DisplayForm,
    template=view.template('ptah.form:templates/form-display.pt'))

view.register_snippet(
    'form-actions', Form,
    template=view.template('ptah.form:templates/form-actions.pt'))

view.register_snippet(
    'form-widget', Field,
    template=view.template('ptah.form:templates/widget.pt'))

view.register_snippet(
    'form-display-widget', Field,
    template=view.template('ptah.form:templates/widget-display.pt'))



import transaction
from ptah import cms, config

from base import Base


class TestApplicationFactoryRegistration(Base):

    def setUp(self):
        super(TestApplicationFactoryRegistration, self).setUp()

        import ptah.cms
        ptah.cms.ApplicationFactory._sql_get_root.reset()

    def _setup_ptah(self):
        pass

    def _make_app(self):
        global ApplicationRoot
        class ApplicationRoot(cms.ApplicationRoot):
            __type__ = cms.Type('app')

        return ApplicationRoot


    def test_app_factory(self):
        import ptah.cms
        ApplicationRoot = self._make_app()

        self._init_ptah()

        factory = ptah.cms.ApplicationFactory(
            ApplicationRoot, '/test', 'root', 'Root App')

        self.assertTrue(factory.id == 'test')
        self.assertTrue(factory.path == '/test/')
        self.assertTrue(factory.name == 'root')
        self.assertTrue(factory.title == 'Root App')
        self.assertTrue(factory.policy is ptah.cms.ApplicationPolicy)

        self._setRequest(self._makeRequest())

        root = factory(self.request)
        r_uri = root.__uri__

        self.assertTrue(isinstance(root, ApplicationRoot))
        self.assertTrue(root.title == 'Root App')
        self.assertTrue(root.__name__ == 'root')
        self.assertTrue(root.__root_path__ == '/test/')
        self.assertTrue(root.__resource_url__(None, {}) == '/test/')
        self.assertTrue(self.request.root is root)
        transaction.commit()

        root = factory(self.request)
        self.assertEqual(root.__uri__, r_uri)
        transaction.commit()

        root = ptah.resolve(r_uri)
        self.assertEqual(root.__uri__, r_uri)

        factory = ptah.cms.ApplicationFactory(
            ApplicationRoot, '', 'root', 'Root App')
        self.assertTrue(factory.default_root)

    def test_app_factory_mutiple(self):
        import ptah.cms

        factory1 = ptah.cms.ApplicationFactory(
            ApplicationRoot, '/app1', 'app1', 'Root App1')
        factory2 = ptah.cms.ApplicationFactory(
            ApplicationRoot, '/app2', 'app2', 'Root App2')

        root1 = factory1()
        root2 = factory2()

        self.assertTrue(root1.__root_path__ == '/app1/')
        self.assertTrue(root2.__root_path__ == '/app2/')
        self.assertTrue(root1.__uri__ != root2.__uri__)

    def test_app_factory_mutiple_same_name(self):
        import ptah.cms

        factory1 = ptah.cms.ApplicationFactory(
            ApplicationRoot, '/test', 'root', 'Root App1')
        factory2 = ptah.cms.ApplicationFactory(
            ApplicationRoot, '/', 'root', 'Root App2')

        self.assertRaises(config.ConflictError, self._init_ptah)

    def test_app_factory_mutiple_same_applications(self):
        import ptah.cms
        self._init_ptah()

        factory1 = ptah.cms.ApplicationFactory(
            ApplicationRoot, '/app1', 'root', 'Root App')
        factory2 = ptah.cms.ApplicationFactory(
            ApplicationRoot, '/app2', 'root', 'Root App')

        root1 = factory1()
        uri1 = root1.__uri__
        self.assertTrue(root1.__root_path__ == '/app1/')
        self.assertTrue(root1.__resource_url__(None, {}) == '/app1/')
        transaction.commit()

        root2 = factory2()

        self.assertTrue(root2.__uri__ == uri1)
        self.assertTrue(root2.__root_path__ == '/app2/')
        self.assertTrue(root2.__resource_url__(None, {}) == '/app2/')

    def test_app_factory_policy(self):
        import ptah.cms

        class CustomPolicy(ptah.cms.ApplicationPolicy):
            pass

        factory = ptah.cms.ApplicationFactory(
            ApplicationRoot, '/app1', 'root', 'Root App', CustomPolicy)

        root = factory(self._makeRequest())
        self.assertTrue(isinstance(root.__parent__, CustomPolicy))


class TestApplicationFactoryCustom(Base):

    def setUp(self):
        super(TestApplicationFactoryCustom, self).setUp()

        import ptah.cms
        ptah.cms.ApplicationFactory._sql_get_root.reset()

    def tearDown(self):
        config.cleanup_system(self.__class__.__module__)
        super(TestApplicationFactoryCustom, self).tearDown()

    def test_app_factory_custom_app(self):
        import ptah.cms

        class CustomApplication(ptah.cms.ApplicationRoot):

            __type__ = ptah.cms.Type('customapp', 'Custom app')

        CustomApplication.__type__.cls = CustomApplication

        factory = ptah.cms.ApplicationFactory(
            CustomApplication, '/', 'root', 'Root App')

        root = factory()
        self.assertTrue(isinstance(root, CustomApplication))

        u_root = root.__uri__
        transaction.commit()

        root = factory()
        self.assertEqual(root.__uri__, u_root)

    def test_app_factory_custom_app2(self):
        import ptah.cms

        class CustomApplication(ptah.cms.ApplicationRoot):

            __type__ = ptah.cms.Type('customapp', 'Custom app')

        CustomApplication.__type__.cls = CustomApplication

        factory = ptah.cms.ApplicationFactory(
            CustomApplication, '/', 'root', 'Root App')

        root = factory()
        self.assertTrue(isinstance(root, CustomApplication))

        u_root = root.__uri__
        transaction.commit()

        root = factory()
        self.assertEqual(root.__uri__, u_root)



""" layout implementation """
from zope import interface
from pyramid.location import lineage
from pyramid.interfaces import IRequest, IRouteRequest

from ptah import config
from ptah.view.base import View
from ptah.view.interfaces import ILayout
from ptah.view.customize import LayerWrapper


def query_layout(context, request, name=''):
    """ query named layout for context """
    assert IRequest.providedBy(request), u"must pass in a request object"

    for context in lineage(context):
        layout = request.registry.queryMultiAdapter(
            (context, request), ILayout, name)
        if layout is not None:
            return layout

    return None


def query_layout_chain(context, request, layoutname=''):
    chain = []

    layout = query_layout(context, request, layoutname)
    if layout is None:
        return chain

    chain.append(layout)
    contexts = {layoutname: layout.context}

    while layout is not None:
        if layout.layout in contexts:
            l_context = contexts[layout.layout].__parent__
        else:
            l_context = context

        layout = query_layout(l_context, request, layout.layout)
        if layout is not None:
            chain.append(layout)
            contexts[layout.name] = layout.context

            if layout.layout is None:
                break

    return chain


class LayoutRenderer(object):

    def __init__(self, layout):
        self.layout = layout

    def __call__(self, context, request, content):
        chain = query_layout_chain(context, request, self.layout)

        for layout in chain:
            layout.update()
            content = layout.render(content)

        return content


class Layout(View):
    interface.implements(ILayout)

    name = ''
    template = None

    @property
    def __name__(self):
        return self.name

    def render(self, content, **kwargs):
        if self.template is None:
            return content

        kwargs.update({'view': self,
                       'content': content,
                       'context': self.context,
                       'request': self.request})

        return self.template(**kwargs)


def register_layout(
    name='', context=None, parent='',
    klass=Layout, template = None, route=None, layer=''):

    if not klass or not issubclass(klass, Layout):
        raise ValueError("klass has to inherit from Layout class")

    discriminator = ('ptah.view:layout', name, context, route, layer)

    info = config.DirectiveInfo()
    info.attach(
        config.Action(
            LayerWrapper(register_layout_impl, discriminator),
            (klass, name, context, template, parent, route),
            discriminator = discriminator)
        )


def register_layout_impl(
    cfg, klass, name, context, template, parent, route_name):

    if not parent:
        layout = None
    elif parent == '.':
        layout = ''
    else:
        layout = parent

    # class attributes
    cdict = {'name': name,
             'layout': layout}

    if template is not None:
        cdict['template'] = template

    if issubclass(klass, Layout) and klass is not Layout:
        layout_class = klass
        for attr, value in cdict.items():
            setattr(layout_class, attr, value)
    else:
        layout_class = type(str('Layout<%s>'%name), (Layout,), cdict)

    layout_class.__config_action__ = cfg.action

    # register layout
    request_iface = IRequest
    if route_name is not None:
        request_iface = cfg.registry.getUtility(
            IRouteRequest, name=route_name)

    cfg.registry.registerAdapter(
        layout_class, (context, request_iface), ILayout, name)



import ptah
from pyramid.testing import DummyRequest

from base import Base


class TestIntrospectModule(Base):

    def tearDown(self):
        ptah.config.cleanup_system(self.__class__.__module__)
        super(TestIntrospectModule, self).tearDown()

    def test_introspect_module(self):
        from ptah.manage.manage import CONFIG, PtahManageRoute
        from ptah.manage.introspect import IntrospectModule, Package

        request = DummyRequest()

        ptah.authService.set_userid('test')
        CONFIG['managers'] = ('*',)
        mr = PtahManageRoute(request)
        mod = mr['introspect']

        self.assertIsInstance(mod, IntrospectModule)

        self.assertRaises(KeyError, mod.__getitem__, 'unknown')

        package = mod['ptah']
        self.assertIsInstance(package, Package)

    def test_introspect_view(self):
        from ptah.manage.introspect import IntrospectModule, MainView

        request = DummyRequest()

        mod = IntrospectModule(None, request)

        res = MainView.__renderer__(mod, request)
        self.assertEqual(res.status, '200 OK')

        view = MainView(mod, request)
        view.update()

        self.assertTrue(bool(view.packages))

    def test_introspect_package_view(self):
        from ptah.manage.introspect import IntrospectModule, PackageView

        request = DummyRequest()

        mod = IntrospectModule(None, request)

        for pkg in mod.list_packages():
            package = mod[pkg]

            res = PackageView.__renderer__(package, request)
            self.assertEqual(res.status, '200 OK')

    def test_introspect_routes(self):
        from ptah.manage import introspect
        from ptah.manage.introspect import IntrospectModule, RoutesView

        introspect.exclude = None

        ptah.view.register_route('test-introspect', '/test/introspect')

        global view

        @ptah.view.pview(route='test-introspect')
        def view(request):
            """ """

        self.p_config.add_route('test-route', '/test/')

        self._init_ptah()

        request = DummyRequest()

        mod = IntrospectModule(None, request)
        res = RoutesView.__renderer__(mod, request)

        self.assertIn(
            "Routes <small>registered pyramid routes</small>", res.body)
        self.assertIn('/test/', res.body)
        self.assertIn('/test/introspect', res.body)
        self.assertIn('Route view', res.body)

    def test_introspect_events(self):
        from ptah.manage import introspect
        from ptah.manage.introspect import IntrospectModule, EventsView

        introspect.exclude = None

        global TestEvent, eventHandler1, eventHandler2

        class TestEvent(object):
            ptah.config.event('Test event')

        @ptah.config.subscriber(TestEvent)
        def eventHandler1(ev):
            """ """

        @ptah.config.subscriber(None, TestEvent)
        def eventHandler2(context, ev):
            """ """

        self._init_ptah()

        request = DummyRequest()
        mod = IntrospectModule(None, request)
        res = EventsView.__renderer__(mod, request)

        self.assertIn("Events <small>event declarations</small>", res.body)
        self.assertIn("Test event", res.body)

        request = DummyRequest(
            params={'ev': 'ptah.manage.tests.test_introspect.TestEvent'})
        res = EventsView.__renderer__(mod, request)

        self.assertIn('Event: Test event', res.body)
        self.assertIn('eventHandler1', res.body)
        self.assertIn('eventHandler2', res.body)


class TestSubscriberIntrospect(Base):

    def test_introspect_subscriber_introspect(self):
        from ptah.config import directives
        from ptah.manage.introspect import SubscriberDirective

        @ptah.config.subscriber(TestEvent)
        def eventHandler1(ev):
            """ """

        @ptah.config.subscriber(None, TestEvent)
        def eventHandler2(context, ev):
            """ """

        data = directives.scan(self.__class__.__module__, set())

        actions = []
        for action in data:
            if action.discriminator[0] == 'ptah.config:subscriber':
                actions.append(action)

        ti = SubscriberDirective(self.request)
        res = ti.renderActions(*actions)

        self.assertIn('ptah.manage.tests.test_introspect.eventHandler1', res)
        self.assertIn('ptah.manage.tests.test_introspect.eventHandler2', res)


class TestRouteIntrospect(Base):

    def test_introspect_route_introspect(self):
        from ptah.config import directives
        from ptah.manage.introspect import RouteDirective

        ptah.view.register_route('test-introspect', '/test/introspect')
        data = directives.scan(self.__class__.__module__, set())

        actions = []
        for action in data:
            if action.discriminator[0] == 'ptah.view:route':
                actions.append(action)

        ti = RouteDirective(self.request)
        res = ti.renderActions(*actions)

        self.assertIn('test-introspect: /test/introspect', res)


class TestViewIntrospect(Base):

    def test_introspect_view_introspect(self):
        from ptah.config import directives
        from ptah.manage.introspect import ViewDirective

        ptah.view.register_route('test-introspect', '/test/introspect')

        global view1, view2, view3

        @ptah.view.pview(route='test-introspect')
        def view1(request):
            """ """

        @ptah.view.pview('view2.html', route='test-introspect')
        def view2(request):
            """ """

        class view3(ptah.view.View):
            ptah.view.pview('view3.html')

        data = directives.scan(self.__class__.__module__, set())

        actions = []
        for action in data:
            if action.discriminator[0] == 'ptah.view:view':
                actions.append(action)

        ti = ViewDirective(self.request)
        res = ti.renderActions(*actions)

        self.assertIn('route: "test-introspect"', res)
        self.assertIn('view: "view2.html" route: "test-introspect"', res)
        self.assertIn('view: view3.html', res)



import colander
import unittest

from colander.tests import TestMapping, TestSequence, TestSchemaNode


class TestptahMapping(TestMapping):

    def _makeOne(self, *arg, **kw):
        from ptah.config.schema import Mapping
        return Mapping(*arg, **kw)

    def test_schema_mapping_flatten_without_name(self):
        node = colander.SchemaNode(
            self._makeOne(),

            colander.SchemaNode(
                colander.Int(),
                name = 'a'),

            colander.SchemaNode(
                colander.Int(),
                name = 'b'),
            )

        result = node.flatten({'a': 1, 'b': 2})
        self.assertEqual(result, {'a': 1, 'b': 2})

    def test_schema_mapping_unflatten_without_name(self):
        node = colander.SchemaNode(
            self._makeOne(),

            colander.SchemaNode(
                self._makeOne(),

                colander.SchemaNode(
                    colander.Int(),
                    name = 'a'),

                colander.SchemaNode(
                    colander.Int(),
                    name = 'b'),

                name = 'node',
                ),
            )

        result = node.unflatten({'node.a': 1, 'node.b': 2})
        self.assertEqual(result, {'node': {'a': 1, 'b': 2}})

    def test_schema_mapping_unflatten_non_existing(self):
        node = colander.SchemaNode(
            self._makeOne(),

            colander.SchemaNode(
                self._makeOne(),
                colander.SchemaNode(colander.Int(), name = 'a'),
                colander.SchemaNode(colander.Int(), name = 'b'),
                name = 'node',
                )
            )

        result = node.unflatten({'node.a': 1, 'node.b': 2, 'node.c': 3})
        self.assertEqual(result, {'node': {'a': 1, 'b': 2}})


class TestptahSequence(TestSequence):

    def _makeOne(self, **kw):
        from ptah.config.schema import Sequence
        return Sequence(**kw)

    def test_schema_sequence_flatten(self):
        node = colander.SchemaNode(
            self._makeOne(),
            colander.SchemaNode(colander.Int()),
            name = 'node')

        result = node.flatten([1, 2])
        self.assertEqual(result, {'node.0': 1, 'node.1': 2})


class TestptahString(unittest.TestCase):

    def test_schema_str(self):
        from ptah.config.schema import Str

        typ = Str()

        self.assertEqual(typ.encoding, 'utf-8')


class TestptahSchemaNode(unittest.TestCase):

    def _makeOne(self, *arg, **kw):
        from ptah.config.schema import SchemaNode
        return SchemaNode(*arg, **kw)

    def test_schema_node_deserialize(self):
        from ptah.config.schema import Required

        node = self._makeOne(
            colander.Int(),
            name = 'node')

        self.assertEqual(node.deserialize('1'), 1)

        self.assertRaises(
            Required, node.deserialize)

        node = self._makeOne(
            colander.Int(),
            name = 'node',
            default = 100)

        self.assertEqual(node.deserialize(), 100)

    def test_schema_node_deserialize_common(self):
        def validator(node, val):
            if val == 10:
                raise colander.Invalid('Error')

        node = self._makeOne(
            colander.Int(),
            name = 'node',
            validator = validator)

        self.assertRaises(
            colander.Invalid, node.deserialize, '10')

        node = self._makeOne(
            colander.Int(),
            name = 'node',
            preparer = lambda x: x * 2,
            default = 100)

        self.assertEqual(node.deserialize(), 200)

    def test_schema_node_deserialize_str(self):
        from ptah.config.schema import Required

        node = self._makeOne(
            colander.Str(),
            name = 'node',
            default = 'default')

        self.assertEqual(node.deserialize('test'), 'test')
        self.assertEqual(node.deserialize(''), 'default')
        self.assertEqual(node.deserialize(), 'default')

        node = self._makeOne(
            colander.Str(),
            name = 'node',
            required = True,
            default = 'default')

        self.assertEqual(node.deserialize('test'), 'test')
        self.assertRaises(Required, node.deserialize, '')
        self.assertRaises(Required, node.deserialize)


class TestRequiredWithDependency(unittest.TestCase):

    def test_schema_required_validator(self):
        from ptah import config
        from ptah.config.schema import Required

        v = config.RequiredWithDependency(
            'field', 'depends', 'depvalue', 'default')

        self.assertEqual(v(None, {}), None)
        self.assertEqual(v(None, {'depends': 'sothing diff'}), None)
        self.assertEqual(v(None,
                           {'field': 'val', 'depends': 'sothing diff'}), None)

        self.assertRaises(
            Required,
            v, {'field':
                colander.SchemaNode(colander.Str(), name='field')},
            {'depends': 'depvalue'})

        v = config.RequiredWithDependency('field', 'depends')
        self.assertRaises(
            Required,
            v, {'field':
                colander.SchemaNode(colander.Str(), name='field')},
            {'depends': 'somthing diff'})

        self.assertEqual(
            v({'field':
               colander.SchemaNode(colander.Str(), name='field')},
              {'field': 'val', 'depends': 'somthing diff'}), None)



""" base class """
import unittest
from ptah import config
from pyramid import testing


class Base(unittest.TestCase):

    def _init_ptah(self, settings={}, handler=None, *args, **kw):
        config.initialize(self.p_config, ('ptah', self.__class__.__module__),
                          initsettings=False)
        config.initialize_settings(settings, self.p_config)

    def _setup_pyramid(self):
        self.request = request = testing.DummyRequest()
        request.params = {}
        self.p_config = testing.setUp(request=request)
        self.p_config.get_routes_mapper()
        self.registry = self.p_config.registry
        self.request.registry = self.p_config.registry

    def _setup_ptah(self):
        config.initialize(
            self.p_config, ('ptah.view', self.__class__.__module__),
            initsettings=False)

    def setUp(self):
        self._setup_pyramid()
        self._setup_ptah()

    def tearDown(self):
        config.cleanup_system(self.__class__.__module__)
        sm = self.p_config
        sm.__init__('base')
        testing.tearDown()



import threading
from datetime import timedelta
from ptah import config, form
from pyramid.interfaces import INewRequest

from ptah import token


class ThreadLocalManager(threading.local):

    def __init__(self):
        self.data = {}

    def get(self, key, default=None):
        return self.data.get(key, default)

    def set(self, key, value):
        self.data[key] = value

    def clear(self):
        self.data = {}

tldata = ThreadLocalManager()


@config.subscriber(INewRequest)
def resetThreadLocalData(ev):
    tldata.clear()


class CSRFService(object):
    """ csrf service for ptah.form """

    TOKEN_TYPE = token.TokenType(
        '1c49d2aacf844557a7aff3dbf09c0740', timedelta(minutes=30))

    def generate(self, data):
        t = token.service.get_bydata(self.TOKEN_TYPE, data)
        if t is not None:
            return t
        return token.service.generate(self.TOKEN_TYPE, data)

    def get(self, t):
        return token.service.get(t)

    def remove(self, t):
        return token.service.remove(t)

form.setCsrfUtility(CSRFService())


class Pagination(object):
    """ simple pagination """

    def __init__(self, page_size, left_neighbours=3, right_neighbours=3):
        self.page_size = page_size
        self.left_neighbours = left_neighbours
        self.right_neighbours = right_neighbours

    def offset(self, current):
        return (current - 1) * self.page_size, self.page_size

    def __call__(self, total, current):
        if not current:
            raise ValueError(current)

        size = int(round(total / float(self.page_size) + 0.4))

        pages = []

        first = 1
        last = size

        prevIdx = current - self.left_neighbours
        nextIdx = current + 1

        if first < current:
            pages.append(first)
        if first + 1 < prevIdx:
            pages.append(None)
        for i in range(prevIdx, prevIdx + self.left_neighbours):
            if first < i:
                pages.append(i)

        pages.append(current)

        for i in range(nextIdx, nextIdx + self.right_neighbours):
            if i < last:
                pages.append(i)
        if nextIdx + self.right_neighbours < last:
            pages.append(None)
        if current < last:
            pages.append(last)

        # prev/next idx
        prevLink = None if current <= 1 else current - 1
        nextLink = None if current >= size else current + 1

        return pages, prevLink, nextLink


@config.cleanup
def cleanup():
    tldata.clear()



""" layout tests """
import os, tempfile, shutil
from zope import interface
from pyramid.interfaces import IRouteRequest

from ptah import view
from ptah.view.interfaces import ILayout
from ptah.view.layout import Layout, LayoutRenderer, \
    query_layout, query_layout_chain

from base import Base


class TestLayout(Base):

    def setUp(self):
        Base.setUp(self)
        self.dir = tempfile.mkdtemp()

    def tearDown(self):
        Base.tearDown(self)
        shutil.rmtree(self.dir)

    def _setup_ptah(self):
        pass

    def test_layout_register_class_errors(self):
        self.assertRaises(ValueError, view.register_layout, 'test',klass=None)

        class LayoutCls(object):
            pass
        self.assertRaises(ValueError, view.register_layout, 'test',klass=LayoutCls)

    def test_layout_register_simple(self):
        view.register_layout('test')
        self._init_ptah()

        layout = self.registry.getMultiAdapter(
            (object(), self.request), ILayout, 'test')

        self.assertEqual(layout.name, 'test')
        self.assertEqual(layout.__name__, 'test')
        self.assertTrue(isinstance(layout, Layout))
        self.assertTrue(issubclass(layout.__class__, Layout))
        self.assertTrue(layout.__class__ is not Layout)

    def test_layout_register_custom_class(self):
        class MyLayout(view.Layout):
            pass

        view.register_layout('test', klass=MyLayout)
        self._init_ptah()

        layout = self.registry.getMultiAdapter(
            (object(), self.request), ILayout, 'test')

        self.assertTrue(isinstance(layout, MyLayout))

    def test_layout_simple_view(self):
        class View(view.View):
            def __call__(self):
                return 'View: test'
        class Layout(view.Layout):
            def render(self, rendered):
                return '<html>%s</html>'%rendered

        view.register_layout('test', klass=Layout)
        self._init_ptah()

        v = View(Context(), self.request)

        # find layout for view
        layout = query_layout(v.context, self.request, 'test')
        self.assertTrue(isinstance(layout, Layout))

        layout.update()
        self.assertEqual(layout.render(v()),
                         '<html>View: test</html>')

    def test_layout_simple_declarative(self):
        global Layout

        class View(view.View):
            def __call__(self):
                return 'View: test'
        class Layout(view.Layout):
            view.layout('test', context=Context)
            def render(self, rendered):
                return '<html>%s</html>'%rendered

        self._init_ptah()

        v = View(Context(), self.request)

        layout = query_layout(v.context, self.request, 'test')
        self.assertTrue(isinstance(layout, Layout))

        layout.update()
        self.assertEqual(layout.render(v()),
                         '<html>View: test</html>')

    def test_layout_simple_notfound(self):
        v = view.View(Context(Context()), self.request)
        layout = query_layout(v.context, self.request, 'test')
        self.assertTrue(layout is None)

    def test_layout_simple_context(self):
        class View(view.View):
            layout = 'test'
            def render(self):
                return 'View: test'
        class Layout(view.Layout):
            def render(self, content):
                return '<html>%s</html>'%content

        view.register_layout('test', klass=Layout, context=Context)
        self._init_ptah()

        v = View(Context(), self.request)

        # find layout for view
        layout = query_layout(v.context, self.request, 'test')
        self.assertTrue(isinstance(layout, Layout))

        layout.update()
        self.assertEqual(layout.render(v.render()), '<html>View: test</html>')

    def test_layout_simple_multilevel(self):
        class Layout(view.Layout):
            def render(self, content):
                return '<html>%s</html>'%content

        view.register_layout('', klass=Layout, context=Root)
        self._init_ptah()

        root = Root()
        context = Context(root)

        # find layout for view
        layout = query_layout(context, self.request, '')
        self.assertTrue(isinstance(layout, Layout))

        renderer = LayoutRenderer('')
        res = renderer(context, self.request, 'View: test')

        self.assertTrue('<html>View: test</html>' in res)

    def test_layout_simple_view_with_template(self):
        fn = os.path.join(self.dir, 'test.pt')
        tmpl = open(fn, 'wb')
        tmpl.write('<html>${content}</html>')
        tmpl.close()

        view.register_layout('test', template = view.template(fn))
        self._init_ptah()

        renderer = LayoutRenderer('test')

        context = Context()
        res = renderer(context, self.request, 'View: test')

        self.assertTrue('<html>View: test</html>' in res)

    def test_layout_simple_chain_one_level(self):
        class LayoutTest(view.Layout):
            def render(self,content):
                return '<div>%s</div>'%content

        class LayoutPage(view.Layout):
            def render(self, content):
                return '<html>%s</html>'%content

        view.register_layout('', klass=LayoutPage, parent=None)
        view.register_layout('test', klass=LayoutTest, parent='.')
        self._init_ptah()

        context = Context()
        renderer = LayoutRenderer('test')
        res = renderer(context, self.request, 'View: test')

        self.assertTrue('<html><div>View: test</div></html>' in res)

    def test_layout_simple_chain_multi_level(self):
        class LayoutTest(view.Layout):
            def render(self, content):
                return '<div>%s</div>'%content

        class LayoutPage(view.Layout):
            def render(self, content):
                return '<html>%s</html>'%content

        view.register_layout('', klass=LayoutPage, context=Root, parent=None)
        view.register_layout('test', klass=LayoutTest, parent='.')
        self._init_ptah()

        root = Root()
        context = Context(root)

        renderer = LayoutRenderer('test')
        res = renderer(context, self.request, 'View: test')

        self.assertTrue('<html><div>View: test</div></html>' in res)

        layout = query_layout(context, self.request, 'test')
        self.assertTrue(isinstance(layout, LayoutTest))

        rootlayout = query_layout(context, self.request, '')
        self.assertTrue(isinstance(rootlayout, LayoutPage))

        rootlayout = query_layout(root, self.request, '')
        self.assertTrue(isinstance(rootlayout, LayoutPage))

    def test_layout_chain_same_layer_id_on_different_levels(self):
        class Layout1(view.Layout):
            def render(self, content):
                return '<div>%s</div>'%content

        class Layout2(view.Layout):
            def render(self, content):
                return '<html>%s</html>'%content

        view.register_layout('', klass=Layout1, context=Context, parent='.')
        view.register_layout('', klass=Layout2, context=Root, parent=None)
        self._init_ptah()

        root = Root()
        context1 = Context2(root)
        context2 = Context(context1)

        renderer = LayoutRenderer('')
        res = renderer(context2, self.request, 'View: test')

        self.assertTrue('<html><div>View: test</div></html>' in res)

    def test_layout_chain_parent_notfound(self):
        class Layout(view.Layout):
            def render(self, content):
                return '<div>%s</div>'%content

        view.register_layout('', klass=Layout, context=Context, parent='page')
        self._init_ptah()

        root = Root()
        context = Context(root)

        renderer = LayoutRenderer('')
        res = renderer(context, self.request, 'View: test')

        self.assertTrue('<div>View: test</div>' in res)

    def test_layout_simple_view_without_template(self):
        class View(view.View):
            def render(self):
                return 'test'

        view.register_layout('test')
        self._init_ptah()

        v = View(Context(), self.request)

        layout = query_layout(v.context, self.request, 'test')
        layout.update()
        self.assertTrue('test' == layout.render(v.render()))

    def test_layout_for_route(self):
        view.register_route('test-route', '/test/', use_global_views=True)
        view.register_layout('test', route = 'test-route')
        self._init_ptah()

        layout = query_layout(Context(), self.request, 'test')
        self.assertIsNone(layout)

        request_iface = self.registry.getUtility(
            IRouteRequest, name='test-route')
        interface.directlyProvides(self.request, request_iface)

        layout = query_layout(Context(), self.request, 'test')
        self.assertIsNotNone(layout)

    def test_layout_chain_multi_level(self):
        class Layout1(view.Layout):
            """ """
        class Layout2(view.Layout):
            """ """
        class Layout3(view.Layout):
            """ """

        class Context1(object):
            """ """
        class Context2(object):
            """ """
        class Context3(object):
            """ """

        view.register_layout('l1', klass=Layout1, context=Context1)
        view.register_layout('l1', klass=Layout2, context=Context2, parent='l1')
        view.register_layout('l3', klass=Layout3, context=Context3, parent='l1')
        self._init_ptah()

        root = Root()
        context1 = Context1()
        context2 = Context2()
        context3 = Context3()

        context1.__parent__ = root
        context2.__parent__ = context1
        context3.__parent__ = context2

        chain = query_layout_chain(context1, self.request, 'l1')

        self.assertEqual(len(chain), 1)
        self.assertIsInstance(chain[0], Layout1)

        chain = query_layout_chain(context2, self.request, 'l1')

        self.assertEqual(len(chain), 2)
        self.assertIsInstance(chain[0], Layout2)
        self.assertIsInstance(chain[1], Layout1)

        chain = query_layout_chain(context3, self.request, 'l3')

        self.assertEqual(len(chain), 3)
        self.assertIsInstance(chain[0], Layout3)
        self.assertIsInstance(chain[1], Layout2)
        self.assertIsInstance(chain[2], Layout1)


class Context(object):

    def __init__(self, parent=None):
        self.__parent__ = parent


class Context2(object):

    def __init__(self, parent=None):
        self.__parent__ = parent


class Root(Context):
    pass



""" message tests """
import os
from zope import interface
from pyramid.interfaces import IRequest

from ptah import view
from ptah.view.interfaces import IMessage, IStatusMessage

from base import Base


class TestStatusMessages(Base):

    def _setup_ptah(self):
        pass

    def test_messages_service(self, skip=False):
        if not skip:
            self._init_ptah()

        service = IStatusMessage(self.request)
        self.assertTrue(IStatusMessage.providedBy(service))
        self.assertEqual(service.messages(), ())

        # add simple msg
        service.add('Test')
        msgs = service.messages()
        self.assertTrue(len(msgs) == 1)
        self.assertTrue(u'Test' in msgs[0])

        # only one message
        service.add('Test')
        msgs = service.messages()
        self.assertTrue(len(msgs) == 1)

        # clear
        msgs = service.clear()
        self.assertTrue(len(msgs) == 1)
        self.assertTrue(u'Test' in msgs[0])
        self.assertEqual(service.messages(), ())

        self.assertEqual(service.clear(), ())

    def test_messages_addmessage(self):
        self._init_ptah()

        service = IStatusMessage(self.request)

        # add_message
        view.add_message(self.request, 'message')

        self.assertEqual(['\n'.join(service.clear()[0].split(os.linesep))],
                         [u'<div class="alert-message info">\n  <a class="close" href="#">\xd7</a>\n  <p>message</p>\n</div>\n'])
        self.assertEqual(service.clear(), ())

    def test_messages_service_no_session(self):
        self._init_ptah()

        del self.request.session

        self.test_messages_service(True)

    def test_messages_warning_msg(self):
        self._init_ptah()

        service = IStatusMessage(self.request)

        # add simple msg
        service.add('warning', 'warning')
        self.assertEqual(
           ['\n'.join(service.clear()[0].split(os.linesep))],
           [u'<div class="alert-message warning">\n  <a class="close" href="#">\xd7</a>\n  <p>warning</p>\n</div>\n'])

    def test_messages_error_msg(self):
        self._init_ptah()

        try:
            raise ValueError('Test')
        except:
            pass

        service = IStatusMessage(self.request)

        service.add('error', 'error')
        self.assertEqual(
            ['\n'.join(service.clear()[0].split(os.linesep))],
            [u'<div class="alert-message error">\n  <a class="close" href="#">\xd7</a>\n  <p>error</p>\n</div>\n'])

        service.add(ValueError('Error'), 'error')
        self.assertEqual(
            ['\n'.join(service.clear()[0].split(os.linesep))],
            [u'<div class="alert-message error">\n  <a class="close" href="#">\xd7</a>\n  <p>ValueError: Error</p>\n</div>\n'])

    def test_messages_custom_msg(self):
        class CustomMessage(object):
            interface.implements(IMessage)

            def __init__(self, request):
                self.request = request

            def render(self, message):
                return '<div class="customMsg">%s</div>'%message

        self._init_ptah()

        sm = self.registry
        sm.registerAdapter(CustomMessage, (IRequest,), IMessage, name='custom')

        service = IStatusMessage(self.request)

        service.add('message', 'custom')
        self.assertEqual(
            service.clear(),
            [u'<div class="customMsg">message</div>'])

    def test_messages_render(self):
        self._init_ptah()

        view.add_message(self.request, 'message')
        msg = view.render_messages(self.request)
        self.assertEqual(
           '\n'.join(msg.split(os.linesep)),
           u'<div class="alert-message info">\n  <a class="close" href="#">\xd7</a>\n  <p>message</p>\n</div>\n')
        self.assertEqual(type(msg), unicode)

        msg = view.render_messages(self.request)
        self.assertEqual(msg, '')

    def test_messages_View(self):
        self._init_ptah()

        v = view.View(None, self.request)

        v.message('message')

        service = IStatusMessage(self.request)

        self.assertEqual(
            ['\n'.join(service.messages()[0].split(os.linesep))],
            [u'<div class="alert-message info">\n  <a class="close" href="#">\xd7</a>\n  <p>message</p>\n</div>\n'])
        self.assertEqual(
            '\n'.join(v.render_messages().split(os.linesep)),
            u'<div class="alert-message info">\n  <a class="close" href="#">\xd7</a>\n  <p>message</p>\n</div>\n')
        self.assertEqual(service.messages(), ())



""" formatter tests """
import colander, pytz
from datetime import datetime, timedelta
from ptah import formatter

from base import Base


class TestFormatter(Base):

    def setUp(self):
        self._setup_pyramid()
        self._init_ptah()

    def test_formatter_registration(self):
        def simple(v):
            return 'simple-%s'%v

        format = formatter.format
        format['simple'] = simple

        self.assertEqual(format.simple('test'), 'simple-test')
        self.assertRaises(ValueError, format.__setitem__, 'simple', simple)

        del format['simple']
        self.assertEqual(getattr(format, 'simple', None), None)
        self.assertFalse('simple' in format)

    def test_timezone_schema_serialize(self):
        typ = formatter.Timezone()

        self.assertTrue(typ.serialize(None, colander.null) is colander.null)
        self.assertEqual(typ.serialize(None, pytz.UTC), 'UTC')

    def test_timezone_schema_deserialize(self):
        typ = formatter.Timezone()

        self.assertTrue(typ.deserialize(None, colander.null) is colander.null)
        self.assertTrue(typ.deserialize(None, '') is colander.null)

        # special case for 'GMT+X' timezones
        self.assertEqual(repr(typ.deserialize(None, 'GMT+6')),
                         "<StaticTzInfo 'Etc/GMT+6'>")

        # general timezones
        self.assertEqual(repr(typ.deserialize(None, 'US/Central')),
                         "<DstTzInfo 'US/Central' CST-1 day, 18:00:00 STD>")

        self.assertEqual(repr(typ.deserialize(None, 'us/central')),
                         "<DstTzInfo 'US/Central' CST-1 day, 18:00:00 STD>")

        # unknown timezone
        self.assertRaises(colander.Invalid, typ.deserialize, None, 'unknown')

    def test_datetime_formatter(self):
        format = formatter.format
        self.assertTrue('datetime' in format)

        # format only datetime
        self.assertEqual(format.datetime('text string'), 'text string')

        # default format
        dt = datetime(2011, 2, 6, 10, 35, 45, 80, pytz.UTC)
        self.assertEqual(format.datetime(dt, 'short'),
                         '02/06/11 04:35 AM')
        self.assertEqual(format.datetime(dt, 'medium'),
                         'Feb 06, 2011 04:35 AM')
        self.assertEqual(format.datetime(dt, 'long'),
                         'February 06, 2011 04:35 AM -0600')
        self.assertEqual(format.datetime(dt, 'full'),
                         'Sunday, February 06, 2011 04:35:45 AM CST')

        # datetime without timezone
        dt = datetime(2011, 2, 6, 10, 35, 45, 80)
        self.assertEqual(format.datetime(dt, 'short'),
                         '02/06/11 04:35 AM')

        # different format
        dt = datetime(2011, 2, 6, 10, 35, 45, 80, pytz.UTC)
        formatter.FORMAT['date_short'] = '%b %d, %Y'

        self.assertEqual(format.datetime(dt, 'short'),
                         'Feb 06, 2011 04:35 AM')

    def test_timedelta_formatter(self):
        format = formatter.format
        self.assertTrue('timedelta' in format)

        # format only timedelta
        self.assertEqual(format.timedelta('text string'), 'text string')

        # full format
        td = timedelta(hours=10, minutes=5, seconds=45)

        self.assertEqual(format.timedelta(td, 'full'),
                         '10 hour(s) 5 min(s) 45 sec(s)')

        # medium format
        self.assertEqual(format.timedelta(td, 'medium'),
                         '10:05:45')

        # seconds format
        self.assertEqual(format.timedelta(td, 'seconds'),
                         '36345.0000')

        # default format
        self.assertEqual(format.timedelta(td), '10:05:45')

    def test_size_formatter(self):
        format = formatter.format
        self.assertTrue('size' in format)

        # format only timedelta
        self.assertEqual(format.size('text string'), 'text string')

        v = 1024
        self.assertEqual(format.size(v, 'b'), '1024 B')

        self.assertEqual(format.size(v, 'k'), '1.00 Kb')

        self.assertEqual(format.size(1024*768, 'm'), '0.75 Mb')
        self.assertEqual(format.size(1024*768*768, 'm'), '576.00 Mb')



import ptah
from pyramid.testing import DummyRequest

from base import Base


class TestRestInspectorModule(Base):

    def test_rest_module(self):
        from ptah.manage.rest import RestModule
        from ptah.manage.manage import CONFIG, PtahManageRoute

        request = DummyRequest()

        ptah.authService.set_userid('test')
        CONFIG['managers'] = ('*',)
        mr = PtahManageRoute(request)
        mod = mr['rest']

        self.assertIsInstance(mod, RestModule)

    def test_view_view(self):
        from ptah.manage.rest import RestModule, RestModuleView

        request = DummyRequest()

        mod = RestModule(None, request)

        res = RestModuleView.__renderer__(mod, request)
        self.assertEqual(res.status, '200 OK')

        view = RestModuleView(mod, request)
        view.update()

        self.assertEqual(
            view.appurl, request.application_url)
        self.assertEqual(
            view.url, '%s/__rest__/cms/' % request.application_url)



from pyramid.testing import DummyRequest
from pyramid.httpexceptions import HTTPFound

from base import Base


class TestSourceView(Base):

    def test_source(self):
        from ptah.manage.source import SourceView

        request = DummyRequest()
        res = SourceView.__renderer__(None, request)
        self.assertIsInstance(res, HTTPFound)
        self.assertEqual(res.headers['location'], '.')

    def test_source_view(self):
        from ptah.manage.source import SourceView

        request = DummyRequest(
            params = {'pkg': 'ptah.view.customize'})

        res = SourceView.__renderer__(None, request)
        self.assertIn('Source: ptah/customize.py', res.body)

    def test_source_view_unknown(self):
        from ptah.manage.source import SourceView

        request = DummyRequest(
            params = {'pkg': 'unknown'})

        res = SourceView.__renderer__(None, request)
        self.assertIsInstance(res, HTTPFound)
        self.assertEqual(res.headers['location'], '.')



""" Form and Field Interfaces """
from zope import interface
from zope.interface.common import mapping
from translationstring import TranslationStringFactory

MessageFactory = _ = TranslationStringFactory('ptah.form')

FORM_INPUT = 'form-input'
FORM_DISPLAY = 'form-display'


class Invalid(Exception):

    def __init__(self, field, msg):
        self.field = field
        self.msg = msg

    def __str__(self):
        return 'Invalid: %s: <%s>' % (self.field, self.msg)

    def __repr__(self):
        return 'Invalid(%s: <%s>)' % (self.field, self.msg)


class _null(object):
    """ Represents a null value in field-related operations. """

    def __nonzero__(self):
        return False

    def __repr__(self):
        return '<widget.null>'

null = _null()


class _required(object):
    """ Represents a required value in field-related operations. """

    def __nonzero__(self):
        return False

    def __repr__(self):
        return '<widget.required>'

required = _required()


# ----[ Vocabulary ]----------------------------------------------------------

# vocabulary/term interfaces
class ITerm(interface.Interface):
    """ term """

    value = interface.Attribute('Value')
    token = interface.Attribute('Token')
    title = interface.Attribute('Title')


class IVocabulary(interface.Interface):
    """ vocabulary """

    def get_term(value):
        """Return an ITitledTokenizedTerm object for the given value

        LookupError is raised if the value isn't in the source
        """

    def get_term_bytoken(token):
        """Return an ITokenizedTerm for the passed-in token.

        If `token` is not represented in the vocabulary, `LookupError`
        is raised.
        """

    def get_value(token):
        """Return a value for a given identifier token

        LookupError is raised if there isn't a value in the source.
        """

    def __iter__():
        """Iterate over terms."""

    def __len__():
        """Return number of terms."""

    def __contains__(value):
        """Check wether terms containes the ``value``."""


# ----[ Widgets ]------------------------------------------------------------

class IWidget(interface.Interface):
    """A widget within a form"""

    node = interface.Attribute('Schema node')
    typ = interface.Attribute('Schema type')

    id = interface.Attribute('Id')
    name = interface.Attribute('Name')
    label = interface.Attribute('Label')
    description = interface.Attribute('Description')
    mode = interface.Attribute('Mode')
    required = interface.Attribute('Required')
    error = interface.Attribute('Error')
    form_value = interface.Attribute('Form Value')
    params = interface.Attribute('Request params')
    localizer = interface.Attribute('Localizer')

    template = interface.Attribute('Template')
    widgetTemplate = interface.Attribute('Widget template')

    value = interface.Attribute('Widget value')

    def extract(default=null):
        """Extract the string value(s) of the widget from the form.

        The return value may be any Python construct, but is typically a
        simple string, sequence of strings or a dictionary.

        The value *must not* be converted into a native format.

        If an error occurs during the extraction, the default value should be
        returned. Since this should never happen, if the widget is properly
        designed and used, it is okay to NOT raise an error here, since we do
        not want to crash the system during an inproper request.

        If there is no value to extract, the default is to be returned.
        """

    def update(request):
        """Setup all of the widget information used for displaying."""

    def render(request):
        """Render form widget. First it tring to use template. If template is
        not set then it uses one of the snippets."""

    def serialize(value):
        """ serialize value into widget compatible form """

    def deserialize(value):
        """ deserialize value """


class ISequenceWidget(IWidget):
    """Term based sequence widget base.

    The sequence widget is used for select items from a sequence. Don't get
    confused, this widget does support to choose one or more values from a
    sequence. The word sequence is not used for the schema field, it's used
    for the values where this widget can choose from.

    This widget base class is used for build single or sequence values based
    on a sequence which is in most use case a collection. e.g.
    IList of IChoice for sequence values or IChoice for single values.

    See also the MultiWidget for build sequence values based on none collection
    based values. e.g. IList of ITextLine
    """

    noValueToken = interface.Attribute('NO_VALUE Token')
    terms = interface.Attribute('Terms')

    def updateTerms():
        """Update the widget's ``terms`` attribute and return the terms.

        This method can be used by external components to get the terms
        without having to worry whether they are already created or not.
        """


class IWidgets(mapping.IEnumerableMapping):
    """A widget manager"""

    prefix = interface.Attribute('Prefix')
    mode = interface.Attribute('Mode')
    errors = interface.Attribute('Errors')

    def update():
        """Setup widgets."""

    def extract(setErrors=True):
        """Extract the values from the widgets and validate them."""


# ----[ Buttons ]------------------------------------------------------------

class IButton(interface.Interface):
    """A button in a form."""

    accessKey = interface.Attribute('Access Key')
    actype = interface.Attribute('Action type (i.e. primary, danger, etc)')
    condition = interface.Attribute('Callable, first argument is form instance')


class IButtons(mapping.IEnumerableMapping):
    """Buttons manager."""


class IActions(mapping.IEnumerableMapping):
    """A button widgets manager"""

    prefix = interface.Attribute('Prefix')

    def update():
        """Setup button widgets."""

    def execute():
        """Execute selected button action."""


# ----[ Forms ]--------------------------------------------------------------

class IForm(interface.Interface):
    """Form"""

    mode = interface.Attribute('Mode')
    widgets = interface.Attribute('Widgets')
    label = interface.Attribute('Label')
    prefix = interface.Attribute('Prefix')
    fields = interface.Attribute('Fields')
    buttons = interface.Attribute('Buttons')

    def form_params():
        '''Return the request params dict.'''

    def form_content():
        '''Return the content to be displayed and/or edited.'''

    def update_widgets():
        '''Update the widgets for the form.

        This method is commonly called from the ``update()`` method and is
        mainly meant to be a hook for subclasses.
        '''

    def validate(data, errors):
        ''' Do form level validation, like schema invariants, etc.

        Add errors to errors (IErrors) object.
        '''

    def extract(setErrors=True):
        '''Extract the data of the form.

        setErrors: needs to be passed to extract() and to sub-widgets'''

    def update():
        '''Update the form.'''

    def render():
        '''Render the form.'''


class IDisplayForm(IForm):
    """Mark a form as display form, used for templates."""


class IInputForm(interface.Interface):
    """A form that is meant to process the input of the form controls."""

    id = interface.Attribute('Id')
    name = interface.Attribute('Name')
    method = interface.Attribute('Method')
    action = interface.Attribute('Action')
    enctype = interface.Attribute('Encoding Type')
    accept = interface.Attribute('Accepted Content Types')
    acceptCharset = interface.Attribute('Accepted Character Sets')

    csrf = interface.Attribute('Enable csrf protection')
    csrfname = interface.Attribute('csrf field name')
    token = interface.Attribute('csrf token')


# --- CSRF ---

class ICSRFService(interface.Interface):
    """ Form csrf protection """

    def generate(data):
        """ Generate csrf token. """

    def get(token):
        """ Get data for csrf token. This is form specific,
        but usually it is form id."""

    def remove(token):
        """ Remove token."""



""" security ptah module """
import ptah
from ptah import view, manage


class PermissionsModule(manage.PtahModule):
    __doc__ = u'A listing of all permission sets and their definitions'

    title = 'Permissions'
    manage.module('permissions')


view.register_snippet(
    'ptah-module-actions', PermissionsModule,
    template = view.template('ptah.manage:templates/permissions-actions.pt'))


class PermissionsView(view.View):
    view.pview(
        context=PermissionsModule,
        template=view.template('ptah.manage:templates/permissions.pt'))

    def update(self):
        self.manage_url = manage.CONFIG.manage_url
        self.permissions = ptah.get_permissions().values()
        self.permissions.sort(key = lambda p: p.title)

        acls = ptah.get_acls()
        self.acls = [acl for acl in acls.values() if acl.name != '']
        self.acls.sort(key = lambda a: a.title)
        self.acls.insert(0, ptah.DEFAULT_ACL)


class RolesView(view.View):
    view.pview(
        'roles.html', PermissionsModule,
        template=view.template('ptah.manage:templates/roles.pt'))

    def update(self):
        self.roles = ptah.get_roles().values()
        self.roles.sort(key = lambda r: r.title)


class RoleIntrospection(object):
    """ Role registrations """

    title = 'Role'
    manage.introspection('ptah:role')

    actions = view.template('ptah.manage:templates/directive-role.pt')

    def __init__(self, request):
        self.request = request

    def renderActions(self, *actions):
        return self.actions(
            roles = ptah.get_roles(),
            actions = actions,
            manage_url = manage.CONFIG.manage_url,
            request = self.request)


class PermissionIntrospection(object):
    """ Permission registrations """

    title = 'Permission'
    manage.introspection('ptah:permission')

    actions = view.template('ptah.manage:templates/directive-permission.pt')

    def __init__(self, request):
        self.request = request

    def renderActions(self, *actions):
        return self.actions(
            actions = actions,
            manage_url = manage.CONFIG.manage_url,
            request = self.request)



from ptah import config
from base import Base


class TestCsrf(Base):

    def tearDown(self):
        config.cleanup_system(self.__class__.__module__)
        super(TestCsrf, self).tearDown()

    def test_csrf_service(self):
        from ptah.util import CSRFService

        self._init_ptah()

        csrf = CSRFService()
        t = csrf.generate('test')

        self.assertEqual(csrf.get(t), 'test')
        self.assertEqual(csrf.generate('test'), t)

        csrf.remove(t)
        self.assertEqual(csrf.get(t), None)

        t2 = csrf.generate('test')
        self.assertTrue(t != t2)



import ptah
import unittest

class TestPagination(unittest.TestCase):

    def test_pagination_values(self):
        page = ptah.Pagination(10)

        self.assertEqual(page.page_size, 10)
        self.assertEqual(page.left_neighbours, 3)
        self.assertEqual(page.right_neighbours, 3)

        page = ptah.Pagination(11, 5, 2)
        self.assertEqual(page.page_size, 11)
        self.assertEqual(page.left_neighbours, 5)
        self.assertEqual(page.right_neighbours, 2)

        self.assertEqual(page.offset(3), (page.page_size*2, page.page_size))

    def test_pagination_nb_begin(self):
        page = ptah.Pagination(10, 2, 2)

        self.assertRaises(ValueError, page, 1000, 0)

        self.assertEqual(
            page(1000, 1)[0], [1, 2, 3, None, 100])

        self.assertEqual(
            page(1000, 2)[0], [1, 2, 3, 4, None, 100])

        self.assertEqual(
            page(1000, 3)[0], [1, 2, 3, 4, 5, None, 100])

        self.assertEqual(
            page(1000, 4)[0], [1, 2, 3, 4, 5, 6, None, 100])

        self.assertEqual(
            page(1000, 5)[0], [1, None, 3, 4, 5, 6, 7, None, 100])

        self.assertEqual(
            page(1000, 6)[0], [1, None, 4, 5, 6, 7, 8, None, 100])


    def test_pagination_nb_end(self):
        page = ptah.Pagination(10, 2, 2)

        self.assertEqual(
            page(1000, 95)[0], [1, None, 93, 94, 95, 96, 97, None, 100])

        self.assertEqual(
            page(1000, 96)[0], [1, None, 94, 95, 96, 97, 98, None, 100])

        self.assertEqual(
            page(1000, 97)[0], [1, None, 95, 96, 97, 98, 99, 100])

        self.assertEqual(
            page(1000, 98)[0], [1, None, 96, 97, 98, 99, 100])

        self.assertEqual(
            page(1000, 99)[0], [1, None, 97, 98, 99, 100])

        self.assertEqual(
            page(1000, 100)[0], [1, None, 98, 99, 100])

    def test_pagination_zero_nb(self):
        page = ptah.Pagination(10, 0, 0)

        self.assertEqual(
            page(1000, 1)[0], [1, None, 100])

        self.assertEqual(
            page(1000, 2)[0], [1, 2, None, 100])

        self.assertEqual(
            page(1000, 3)[0], [1, None, 3, None, 100])

    def test_pagination_one_nb(self):
        page = ptah.Pagination(10, 1, 1)

        self.assertEqual(
            page(1000, 1)[0], [1, 2, None, 100])

        self.assertEqual(
            page(1000, 2)[0], [1, 2, 3, None, 100])

        self.assertEqual(
            page(1000, 3)[0], [1, 2, 3, 4, None, 100])

        self.assertEqual(
            page(1000, 4)[0], [1, None, 3, 4, 5, None, 100])

    def test_pagination_prev_next(self):
        page = ptah.Pagination(10, 3, 3)

        self.assertEqual(page(1000, 1)[1], None)
        self.assertEqual(page(1000, 2)[1], 1)
        self.assertEqual(page(1000, 3)[1], 2)
        self.assertEqual(page(1000, 100)[1], 99)

        self.assertEqual(page(1000, 100)[2], None)
        self.assertEqual(page(1000, 99)[2], 100)
        self.assertEqual(page(1000, 98)[2], 99)
        self.assertEqual(page(1000, 1)[2], 2)



""" default actions """
import ptah

ptah.cms.uiaction(
    ptah.cms.Content, **{'id': 'view',
                         'title': 'View',
                         'action': '',
                         'permission': ptah.cms.View,
                         'sort_weight': 0.5})

ptah.cms.uiaction(
    ptah.cms.Content, **{'id': 'edit',
                         'title': 'Edit',
                         'action': 'edit.html',
                         'permission': ptah.cms.ModifyContent,
                         'sort_weight': 0.6})


ptah.cms.uiaction(
    ptah.cms.Container, **{'id': 'adding',
                           'title': 'Add content',
                           'action': '+/',
                           'permission': ptah.cms.AddContent,
                           'sort_weight': 5.0})


ptah.cms.uiaction(
    ptah.ILocalRolesAware, **{'id': 'sharing',
                              'title': 'Sharing',
                              'action': 'sharing.html',
                              'permission': ptah.cms.ShareContent,
                              'sort_weight': 10.0})



# ptah api

# uri
from ptah.uri import resolve
from ptah.uri import resolver
from ptah.uri import register_uri_resolver
from ptah.uri import extract_uri_schema
from ptah.uri import UriFactory

# security
from ptah.authentication import authService
from ptah.authentication import SUPERUSER_URI

from ptah.authentication import auth_checker
from ptah.authentication import register_auth_provider

from ptah.authentication import search_principals
from ptah.authentication import principal_searcher
from ptah.authentication import register_principal_searcher

from ptah.interfaces import IACLsAware
from ptah.interfaces import IOwnersAware
from ptah.interfaces import ILocalRolesAware

# acl
from ptah.security import ACL
from ptah.security import ACLsProperty
from ptah.security import get_acls

# role
from ptah.security import Role
from ptah.security import get_roles
from ptah.security import get_local_roles

# permission
from ptah.security import Permission
from ptah.security import get_permissions
from ptah.security import check_permission

# default roles and permissions
from ptah.security import Everyone
from ptah.security import Authenticated
from ptah.security import Owner
from ptah.security import DEFAULT_ACL
from ptah.security import NOT_ALLOWED
from pyramid.security import NO_PERMISSION_REQUIRED

# password tool
from ptah.password import pwd_tool
from ptah.password import password_changer
from ptah.password import PWD_CONFIG

# formatter
from ptah.formatter import format

# rest api
from ptah.rest import RestService
from ptah.rest import enable_rest_api

# events
from ptah import events

# mail templates
from ptah import mail
from ptah.settings import MAIL

# pagination
from ptah.util import Pagination

# thread local data
from ptah.util import tldata

# ReST renderer
from ptah.rst import rst_to_html

# sqlalchemy utils
from ptah.sqla import QueryFreezer
from ptah.sqla import JsonDictType
from ptah.sqla import JsonListType
from ptah.sqla import generate_fieldset
from ptah.sqla import build_sqla_fieldset

# manage
from ptah import manage

# cms
from ptah import cms


# pyramid include
def includeme(config):
    config.add_directive('ptah_initialize', ptah_initialize)


def make_wsgi_app(global_settings, **settings):
    """ Create wsgi application, this function initialize
    `ptah` and sends :py:class:`AppInitialized` event.
    It is possible to use this function as entry point for paster based
    deployment::

      [app:myapp]
      use = egg:ptah#app

    """
    import sys
    import ptah
    from pyramid.config import Configurator

    authService.set_userid(SUPERUSER_URI)

    # configuration
    config = Configurator(settings=settings)

    # initialization
    packages = settings.get('packages', None)
    autoinclude = settings.get('autoinclude', True)
    try:
        ptah_initialize(config, packages, autoinclude)
    except Exception, e:
        if isinstance(e, ptah.config.StopException):
            print e.print_tb()

        sys.exit(0)
        return

    config.commit()

    # create wsgi app
    return config.make_wsgi_app()


# initialize ptah
def ptah_initialize(configurator, packages=None, autoinclude=False):
    """ Initialize ptah packages.
    Load all ptah packages and intialize ptah settings system.

    This function automatically called by :py:func:`make_wsgi_app` function.
    """
    import ptah
    import sqlahelper
    import transaction
    from pyramid.exceptions import  ConfigurationExecutionError

    configurator.include('pyramid_tm')
    configurator.begin()

    try:
        settings = configurator.registry.settings

        # exclude
        excludes = []
        if 'ptah.excludes' in settings:
            excludes.extend(s.strip()
                            for s in settings['ptah.excludes'].split())

        # load packages
        ptah.config.initialize(
            configurator, packages, excludes, autoinclude, initsettings=True)

        configurator.commit()

        # create sql tables
        Base = sqlahelper.get_base()
        Base.metadata.create_all()

        # send AppStarting event
        ptah.config.start(configurator)

        # commit possible transaction
        transaction.commit()
    except Exception, e:
        if isinstance(e, ConfigurationExecutionError):
            e = e.evalue

        if not isinstance(e, ptah.config.StopException):
            ptah.config.shutdown()
            e = ptah.config.StopException(e)
            raise e

        ptah.config.shutdown()
        raise e
    finally:
        configurator.end()



import transaction
from pyramid.interfaces import ITraverser

import ptah
from ptah import config

from base import Base


class ApplicationRoot(ptah.cms.ApplicationRoot):

    __type__ = ptah.cms.Type('traverserapp')


class TestTraverser(Base):

    def tearDown(self):
        config.cleanup_system(self.__class__.__module__)
        super(TestTraverser, self).tearDown()

    def _create_content(self):
        import ptah.cms

        factory = ptah.cms.ApplicationFactory(
            ApplicationRoot, '/test', 'root', 'Root App')
        self.factory = factory

        class MyContent(ptah.cms.Content):
            __mapper_args__ = {'polymorphic_identity': 'mycontent'}
            __uri_factory__ = ptah.UriFactory('test')

        root = factory()

        folder = MyContent(
            __name__ = 'folder',
            __parent__ = root,
            __path__ = '%sfolder/'%root.__path__)
        self.folder_uri = folder.__uri__

        content = MyContent(
            __name__ = 'content',
            __parent__ = folder,
            __path__ = '%scontent/'%folder.__path__)
        self.content_uri = content.__uri__

        ptah.cms.Session.add(folder)
        ptah.cms.Session.add(content)
        transaction.commit()

    def test_traverser_root_view(self):
        import ptah.cms
        from ptah.cms.traverser import ContentTraverser

        self._create_content()

        request = self._makeRequest(
            {'PATH_INFO': '/test/index.html'})

        root = self.factory(request)

        traverser = ITraverser(root)

        self.assertTrue(isinstance(traverser, ContentTraverser))

        info = traverser(request)
        self.assertTrue(info['context'] is root)
        self.assertEqual(info['view_name'], 'index.html')

    def test_traverser_default_root(self):
        import ptah.cms

        request = self._makeRequest(
            {'PATH_INFO': '/test/index.html',
             'bfg.routes.route': {}})

        factory = ptah.cms.ApplicationFactory(
            ApplicationRoot, '', 'root', 'Root App')
        root = factory(request)

        traverser = ITraverser(root)

        info = traverser(request)
        self.assertTrue(info['context'] is root)
        self.assertEqual(info['view_name'], 'test')

    def test_traverser_root_no_view(self):
        self._create_content()

        request = self._makeRequest({'PATH_INFO': '/test/'})

        root = self.factory(request)
        traverser = ITraverser(root)

        info = traverser(request)
        self.assertTrue(info['context'] is root)
        self.assertEqual(info['view_name'], '')

    def test_traverser_folder(self):
        self._create_content()

        request = self._makeRequest({'PATH_INFO': '/test/folder'})

        root = self.factory(request)
        traverser = ITraverser(root)

        info = traverser(request)
        self.assertEqual(info['context'].__uri__, self.folder_uri)
        self.assertEqual(info['view_name'], '')
        self.assertEqual(info['traversed'], ('folder',))

    def test_traverser_folder_2(self):
        self._create_content()

        request = self._makeRequest({'PATH_INFO': '/test/folder/'})

        root = self.factory(request)
        traverser = ITraverser(root)

        info = traverser(request)
        self.assertEqual(info['context'].__uri__, self.folder_uri)
        self.assertEqual(info['view_name'], '')
        self.assertEqual(info['traversed'], ('folder',))

    def test_traverser_folder_view(self):
        self._create_content()

        request = self._makeRequest({'PATH_INFO': '/test/folder/index.html'})

        root = self.factory(request)
        traverser = ITraverser(root)

        info = traverser(request)
        self.assertEqual(info['context'].__uri__, self.folder_uri)
        self.assertEqual(info['view_name'], 'index.html')
        self.assertEqual(info['traversed'], ('folder',))

    def test_traverser_folder_subcontent1(self):
        self._create_content()

        request = self._makeRequest({'PATH_INFO': '/test/folder/content'})

        root = self.factory(request)
        traverser = ITraverser(root)

        info = traverser(request)
        self.assertEqual(info['context'].__uri__, self.content_uri)
        self.assertEqual(info['view_name'], '')
        self.assertEqual(info['traversed'], ('folder','content'))

    def test_traverser_folder_subcontent2(self):
        self._create_content()

        request = self._makeRequest({'PATH_INFO': '/test/folder/content/'})

        root = self.factory(request)
        traverser = ITraverser(root)

        info = traverser(request)
        self.assertEqual(info['context'].__uri__, self.content_uri)
        self.assertEqual(info['view_name'], '')
        self.assertEqual(info['traversed'], ('folder','content'))

    def test_traverser_folder_subcontent_view(self):
        self._create_content()

        request = self._makeRequest(
            {'PATH_INFO': '/test/folder/content/index.html'})

        root = self.factory(request)
        traverser = ITraverser(root)

        info = traverser(request)
        self.assertEqual(info['context'].__uri__, self.content_uri)
        self.assertEqual(info['view_name'], 'index.html')
        self.assertEqual(info['traversed'], ('folder','content'))

    def test_traverser_folder_subcontent_view2(self):
        self._create_content()

        request = self._makeRequest(
            {'PATH_INFO': '/test//folder//content/index.html'})

        root = self.factory(request)
        traverser = ITraverser(root)

        info = traverser(request)
        self.assertEqual(info['context'].__uri__, self.content_uri)
        self.assertEqual(info['view_name'], 'index.html')
        self.assertEqual(info['traversed'], ('folder','content'))



import ptah
import sqlahelper
import sqlalchemy
from pyramid.testing import DummyRequest
from pyramid.httpexceptions import HTTPNotFound

from base import Base


class TestLayoutPreview(Base):

    def setUp(self):
        try:
            engine = sqlahelper.get_engine()
        except: # pragma: no cover
            engine = sqlalchemy.engine_from_config(
                {'sqlalchemy.url': 'sqlite://'})
            sqlahelper.add_engine(engine)

        self._setup_pyramid()

    def test_layout_preview_notfound(self):
        from ptah.manage.layout import LayoutPreview

        request = DummyRequest()

        v = LayoutPreview(None,request)
        self.assertRaises(HTTPNotFound, v.update)

    def test_layout_preview_notfound2(self):
        from ptah.manage.layout import LayoutPreview

        class Context(object):
            """ """

        global View
        class View(object):
            ptah.view.pview('', Context)
            def render(self):
                return 'test' # pragma: no cover

        self._init_ptah()

        request = DummyRequest()

        v = LayoutPreview(Context(), request)
        self.assertRaises(HTTPNotFound, v.update)

    def test_layout_preview(self):
        from ptah.manage.layout import LayoutPreview

        class Context(object):
            """ """
            __name__ = 'test'

        def template(*args, **kw):
            return 'test'

        global View2
        class View2(ptah.view.View):
            ptah.view.pview('', Context, template=template)

            def update(self):
                """ """

        global Layout
        class Layout(ptah.view.Layout):
            ptah.view.layout('', Context, parent=None)

            def render(self, content):
                return '<div>%s</div>'%content

        self._init_ptah()

        request = DummyRequest()

        v = LayoutPreview(Context(), request)
        v.update()
        res = v.render()

        self.assertIn('<div style="border: 4px solid yellow">', res)
        self.assertIn('ptah.manage.tests.test_layout.Layout', res)
        self.assertIn('ptah.manage.tests.test_layout', res)
        self.assertIn('function template at', res)

    def test_layout_preview2(self):
        from ptah.manage.layout import LayoutPreview

        class Context(object):
            """ """
            __name__ = 'test'

        global View

        @ptah.view.pview('', Context, layout='')
        def View(context, request):
            return 'test' # pragma: no cover

        global Layout
        class Layout(ptah.view.Layout):
            ptah.view.layout('', Context, parent=None)

            template = 'layout template'

            def render(self, content):
                return '<div>%s</div>'%content

        self._init_ptah()

        request = DummyRequest()

        v = LayoutPreview(Context(), request)
        v.update()
        res = v.render()

        self.assertIn('<div style="border: 4px solid yellow">', res)
        self.assertIn('"view-factory": "View"', res)
        self.assertIn('layout template', res)



import unittest
import sqlahelper
import sqlalchemy
import transaction
from pyramid import testing
from pyramid.interfaces import IAuthenticationPolicy, IAuthorizationPolicy
from pyramid.authorization import ACLAuthorizationPolicy
from pyramid.authentication import AuthTktAuthenticationPolicy

import ptah
from ptah import config


class Base(unittest.TestCase):

    _settings = {'sqla.url': 'sqlite://',
                 'sqla.cache': False}

    def _makeRequest(self, environ=None):
        from pyramid.request import Request
        if environ is None:
            environ = self._makeEnviron()
        request = Request(environ)
        request.registry = self.p_config.registry
        return request

    def _makeEnviron(self, **extras):
        environ = {
            'wsgi.url_scheme':'http',
            'wsgi.version':(1,0),
            'PATH_INFO': '/',
            'SCRIPT_NAME': '',
            'SERVER_NAME':'localhost',
            'SERVER_PORT':'8080',
            'REQUEST_METHOD':'GET',
            }
        environ.update(extras)
        return environ

    def _init_ptah(self, settings=None, handler=None, *args, **kw):
        if settings is None:
            settings = self._settings
        config.initialize(
            self.p_config, ('ptah', self.__class__.__module__),
            initsettings = False)
        config.initialize_settings(settings, self.p_config)

        # create sql tables
        Base = sqlahelper.get_base()
        Base.metadata.drop_all()
        Base.metadata.create_all()
        transaction.commit()

    def _setup_pyramid(self):
        self.request = request = testing.DummyRequest()
        request.params = {}
        self.p_config = testing.setUp(request=request)
        self.p_config.get_routes_mapper()
        self.registry = self.p_config.registry
        self.request.registry = self.p_config.registry

        policy = AuthTktAuthenticationPolicy(
            'secret', callback= ptah.get_local_roles)

        self.registry.registerUtility(
            policy, IAuthenticationPolicy)

        self.registry.registerUtility(
            ACLAuthorizationPolicy(), IAuthorizationPolicy)

    def _setup_ptah(self):
        self._init_ptah()

    def _setRequest(self, request):
        self.request = request
        self.p_config.end()
        self.p_config.begin(request)

    def setUp(self):
        try:
            engine = sqlahelper.get_engine()
        except:
            engine = sqlalchemy.engine_from_config(
                {'sqlalchemy.url': 'sqlite://'})
            sqlahelper.add_engine(engine)

        self._setup_pyramid()
        self._setup_ptah()

    def tearDown(self):
        #config.cleanup_system(self.__class__.__module__)
        config.cleanup_system()

        sm = self.p_config
        sm.__init__('base')
        testing.tearDown()

        Session = sqlahelper.get_session()
        Session.expunge_all()



import unittest


class StopExceptionTesting(unittest.TestCase):

    def test_api_stopexception_msg(self):
        from ptah.config import api

        err = api.StopException('Error message')

        self.assertEqual(str(err), '\nError message')
        self.assertEqual(err.print_tb(), 'Error message')

    def test_api_stopexception_exc(self):
        from ptah.config import api

        try:
            raise ValueError('err')
        except Exception, exc:
            pass

        err = api.StopException(exc)

        self.assertIn("raise ValueError('err')", err.print_tb())


class LoadpackageTesting(unittest.TestCase):

    def test_exclude_test(self):
        from ptah.config import api

        self.assertFalse(api.exclude('blah.test'))
        self.assertFalse(api.exclude('blah.ftest'))
        self.assertFalse(api.exclude('blah.subpkg', ('blah.',)))
        self.assertTrue(api.exclude('blah.subpkg'))

    def test_loadpackages(self):
        from ptah.config import api

        self.assertEqual(
            api.list_packages(('ptah',), excludes=('ptah',)), [])

        self.assertIn('ptah', api.list_packages())
        self.assertEqual(api.list_packages(excludes=('ptah',)), [])

    def test_stop_exc(self):
        from ptah.config import api

        err = ValueError('test')

        exc = api.StopException(err)
        self.assertIs(exc.exc, err)
        self.assertEqual(str(exc), '\ntest')



# ptah.view public API

# static resources
from ptah.view.resources import static
from ptah.view.resources import static_url

# resource library
from ptah.view.library import library
from ptah.view.library import include
from ptah.view.library import render_includes


# !!! PRIVATE API !!!

# path/template
from ptah.view.tmpl import path, template

# base view
from ptah.view.base import View

# snippet
from ptah.view.snippet import Snippet
from ptah.view.snippet import snippettype
from ptah.view.snippet import render_snippet
from ptah.view.snippet import register_snippet

# route
from ptah.view.route import register_route

# layout
from ptah.view.layout import Layout
from ptah.view.layout import LayoutRenderer
from ptah.view.layout import query_layout
from ptah.view.layout import query_layout_chain
from ptah.view.layout import register_layout

# view
from ptah.view.view import render_view
from ptah.view.view import register_view
from ptah.view.renderers import set_checkpermission
from ptah.view.renderers import ViewRenderer
from ptah.view.renderers import TemplateRenderer

# layer
from ptah.view.customize import LayerWrapper

# directives
from ptah.view.directives import pview
from ptah.view.directives import layout
from ptah.view.directives import snippet

# status message
from ptah.view.message import Message
from ptah.view.message import add_message
from ptah.view.message import render_messages



##############################################################################
#
# Copyright (c) 2003 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Test of the Vocabulary and related support APIs.
"""
import unittest
from ptah.form import vocabulary


class SimpleVocabularyTests(unittest.TestCase):

    list_vocab = vocabulary.SimpleVocabulary.from_values(1, 2, 3)
    items_vocab = vocabulary.SimpleVocabulary.from_items(
        (1, 'one'), (2, 'two'), (3, 'three'), (4, 'fore!'))

    def test_simple_term(self):
        t = vocabulary.SimpleTerm(1)
        self.assertEqual(t.value, 1)
        self.assertEqual(t.token, "1")
        t = vocabulary.SimpleTerm(1, "One")
        self.assertEqual(t.value, 1)
        self.assertEqual(t.token, "One")

    def test_simple_term_title(self):
        t = vocabulary.SimpleTerm(1)
        self.failUnless(t.title is None)
        t = vocabulary.SimpleTerm(1, title="Title")
        self.failUnlessEqual(t.title, "Title")

    def test_order(self):
        value = 1
        for t in self.list_vocab:
            self.assertEqual(t.value, value)
            value += 1

        value = 1
        for t in self.items_vocab:
            self.assertEqual(t.value, value)
            value += 1

    def test_len(self):
        self.assertEqual(len(self.list_vocab), 3)
        self.assertEqual(len(self.items_vocab), 4)

    def test_contains(self):
        for v in (self.list_vocab, self.items_vocab):
            self.assert_(1 in v and 2 in v and 3 in v)
            self.assert_(5 not in v)

        self.assertNotIn([500], self.items_vocab)

    def test_iter_and_get_term(self):
        for v in (self.list_vocab, self.items_vocab):
            for term in v:
                self.assert_(v.get_term(term.value) is term)
                self.assert_(v.get_term_bytoken(term.token) is term)

    def test_getvalue(self):
        self.assertEqual(self.items_vocab.get_value('one'), 1)
        self.assertRaises(LookupError, self.items_vocab.get_value, 'unknown')

    def test_getterm(self):
        term = self.items_vocab.get_term(1)
        self.assertEqual(term.token, 'one')
        self.assertRaises(LookupError, self.items_vocab.get_term, 500)

    def test_getterm_bytoken(self):
        term = self.items_vocab.get_term_bytoken('one')
        self.assertEqual(term.token, 'one')
        self.assertEqual(term.value, 1)
        self.assertRaises(LookupError,
                          self.items_vocab.get_term_bytoken, 'unknown')

    def test_nonunique_tokens(self):
        self.assertRaises(
            ValueError, vocabulary.SimpleVocabulary.from_values, 2, '2')
        self.assertRaises(
            ValueError, vocabulary.SimpleVocabulary.from_items,
            ('one', 1), ('another one', '1'))
        self.assertRaises(
            ValueError, vocabulary.SimpleVocabulary.from_items,
            ('one', 0), ('one', 1))

    def test_nonunique_token_message(self):
        try:
            vocabulary.SimpleVocabulary.from_values(2, '2')
        except ValueError, e:
            self.assertEquals(str(e), "term tokens must be unique: '2'")

    def test_nonunique_token_messages(self):
        try:
            vocabulary.SimpleVocabulary.from_items(('one', 0), ('one', 1))
        except ValueError, e:
            self.assertEquals(str(e), "term values must be unique: 'one'")

    def test_overriding_create_term(self):
        class MyTerm(object):
            def __init__(self, value):
                self.value = value
                self.token = repr(value)
                self.nextvalue = value + 1

        class MyVocabulary(vocabulary.SimpleVocabulary):
            def create_term(cls, value):
                return MyTerm(value)
            create_term = classmethod(create_term)

        vocab = MyVocabulary.from_values(1, 2, 3)
        for term in vocab:
            self.assertEqual(term.value + 1, term.nextvalue)



from ptah import form
from base import Base

field = form.TextField(
    'test', title = 'Test node')

field1 = form.TextField(
    'test1', title = 'Test node')


class TestFieldset(Base):

    def test_fieldset_name_title(self):
        fieldset = form.Fieldset(field)

        self.assertEqual(fieldset.name, '')
        self.assertEqual(fieldset.legend, '')
        self.assertEqual(fieldset.prefix, '')

        fieldset = form.Fieldset(field1, name='othername', legend='Other name')

        self.assertEqual(fieldset.name, 'othername')
        self.assertEqual(fieldset.legend, 'Other name')

    def test_fieldset_nested(self):
        fieldset = form.Fieldset(
            field,
            form.Fieldset(name='fs', *(field,)))

        self.assertEqual(fieldset['fs'].name, 'fs')
        self.assertEqual(fieldset['fs'].prefix, 'fs.')

    def test_fieldset_fields(self):
        fieldset = form.Fieldset(field, field1)

        self.assertEqual(list(fieldset.fields()), [field, field1])

    def test_fieldset_append_simple(self):
        fieldset = form.Fieldset(field, name='test')

        self.assertIn('test', fieldset)
        self.assertEqual(fieldset.prefix, 'test.')

        self.assertIs(fieldset['test'], field)

        self.assertRaises(ValueError, fieldset.append, field)
        self.assertRaises(TypeError, fieldset.append, object())

    def test_fieldset_append_fieldset(self):
        fieldset = form.Fieldset(field, name='schema')
        self.assertEqual(list(fieldset.fieldsets()), [fieldset])

        fs = form.Fieldset(field1, name='schema2', )

        fieldset.append(fs)
        self.assertEqual(len(list(fieldset.fieldsets())), 2)

        self.assertIn('schema2', fieldset)
        self.assertIs(fieldset['schema2'], fs)
        self.assertRaises(ValueError, fieldset.append, fs)

    def test_fieldset_select(self):
        fieldset = form.Fieldset(field, field1)

        newfs = fieldset.select('test')
        self.assertNotIn('test1', newfs)
        self.assertEqual(newfs.keys(), ['test'])

    def test_fieldset_omit(self):
        fieldset = form.Fieldset(field, field1)

        newfs = fieldset.omit('test')
        self.assertNotIn('test', newfs)
        self.assertEqual(newfs.keys(), ['test1'])

    def test_fieldset_add(self):
        fieldset = form.Fieldset(field)
        fieldset = fieldset + form.Fieldset(field1)

        self.assertIn('test', fieldset)
        self.assertIn('test1', fieldset)
        self.assertEqual(fieldset.keys(), ['test', 'test1'])

    def test_fieldset_iadd(self):
        fieldset = form.Fieldset(field)
        fieldset += form.Fieldset(field1)

        self.assertIn('test', fieldset)
        self.assertIn('test1', fieldset)
        self.assertEqual(fieldset.keys(), ['test', 'test1'])

    def test_fieldset_add_err(self):
        fieldset = form.Fieldset(field)

        self.assertRaises(ValueError, fieldset.__add__, object())

    def test_fieldset_bind(self):
        fieldset = form.Fieldset(field)

        params = object()
        data = {'test': 'CONTENT'}

        fs = fieldset.bind(data, params)

        self.assertIsNot(fieldset, fs)
        self.assertEqual(len(fieldset), len(fs))
        self.assertIs(fs.params, params)
        self.assertIs(fs.data, data)

        self.assertIs(fs['test'].params, params)
        self.assertEqual(fs['test'].value, 'CONTENT')

        fs = fieldset.bind()
        self.assertIsNot(fieldset, fs)
        self.assertEqual(len(fieldset), len(fs))
        self.assertEqual(fs.params, {})
        self.assertEqual(fs.data, {})
        self.assertEqual(fs['test'].params, {})
        self.assertIs(fs['test'].value, form.null)

    def test_fieldset_bind_nested(self):
        fieldset = form.Fieldset(
            field,
            form.Fieldset(name='fs', *(field,)))

        params = object()
        content = {'test': 'CONTENT',
                   'fs': {'test': 'NESTED CONTENT'}}

        fs = fieldset.bind(content, params)

        self.assertEqual(fs['fs']['test'].value, 'NESTED CONTENT')

        fs = fieldset.bind()
        self.assertIs(fs['fs']['test'].value, form.null)

    def test_fieldset_validate(self):
        def validator(fs, data):
            raise form.Invalid(fs, 'msg')

        fieldset = form.Fieldset(field, validator=validator)
        self.assertRaises(form.Invalid, fieldset.validate, {})

        fieldset.bind()
        self.assertRaises(form.Invalid, fieldset.validate, {})

    def _makeOne(self, name, **kw):
        class MyField(form.Field):
            def serialize(self, value): # pragma: no cover
                return value
            def deserialize(self, value):
                return value

        return MyField(name, **kw)

    def test_fieldset_extract_display(self):
        field = self._makeOne('test', mode=form.FORM_DISPLAY)
        fieldset = form.Fieldset(field).bind(None, {'test': 'VALUE'})
        data, errors = fieldset.extract()
        self.assertEqual(data, {})
        self.assertEqual(errors, [])

    def test_fieldset_extract_missing(self):
        field = self._makeOne('test')
        fieldset = form.Fieldset(field).bind()

        data, errors = fieldset.extract()
        self.assertIs(errors[0].field, fieldset['test'])
        self.assertEqual(errors[0].msg, u'Required')

    def test_fieldset_extract_missing_nested(self):
        field = self._makeOne('test')
        fieldset = form.Fieldset(
            field,
            form.Fieldset(name='fs', *(field,))).bind()

        data, errors = fieldset.extract()
        self.assertIs(errors[0].field, fieldset['fs']['test'])
        self.assertIs(errors[1].field, fieldset['test'])
        self.assertEqual(errors[0].msg, u'Required')
        self.assertEqual(errors[1].msg, u'Required')

    def test_fieldset_extract(self):
        field = self._makeOne('test')
        fieldset = form.Fieldset(field).bind(params={'test': 'FORM'})

        data, errors = fieldset.extract()
        self.assertFalse(bool(errors))
        self.assertEqual(data['test'], 'FORM')

    def test_fieldset_extract_nested(self):
        field = self._makeOne('test')
        fieldset = form.Fieldset(
            field,
            form.Fieldset(name='fs', *(field,))
            ).bind(params={'test': 'FORM', 'fs.test': 'NESTED FORM'})

        data, errors = fieldset.extract()
        self.assertFalse(bool(errors))
        self.assertEqual(data['test'], 'FORM')
        self.assertEqual(data['fs']['test'], 'NESTED FORM')

    def test_fieldset_extract_preparer(self):
        def lower(val):
            return val.lower()

        field = self._makeOne('test', preparer=lower)
        fieldset = form.Fieldset(field).bind(params={'test': 'FORM'})

        data, errors = fieldset.extract()
        self.assertEqual(data['test'], 'form')

    def test_fieldset_extract_validate(self):
        def validator(fs, data):
            raise form.Invalid(fs, 'msg')

        field = self._makeOne('test')
        fieldset = form.Fieldset(field, validator=validator)
        fieldset = fieldset.bind(params={'test': 'FORM'})

        data, errors = fieldset.extract()
        self.assertEqual(len(errors), 1)


class TestFieldsetErrors(Base):

    def test_fieldset_errors(self):
        err1 = form.Invalid(field, 'error1')
        err2 = form.Invalid(field1, 'error2')

        fieldset = object()

        errors = form.FieldsetErrors(fieldset, err1, err2)
        self.assertIn(err1, errors)
        self.assertIn(err2, errors)
        self.assertIs(errors.fieldset, fieldset)
        self.assertEqual(errors.msg, {'test': 'error1', 'test1': 'error2'})

        self.assertEqual(str(err1), "Invalid: <TextField 'test'>: <error1>")
        self.assertEqual(repr(err1), "Invalid(<TextField 'test'>: <error1>)")
        self.assertEqual(str(form.null), '<widget.null>')

        self.assertFalse(form.required)
        self.assertEqual(repr(form.required), '<widget.required>')


class TestField(Base):

    def test_field_ctor(self):
        from ptah import form

        field = form.Field('test', **{'title': 'Title',
                                      'description': 'Description',
                                      'readonly': True,
                                      'default': 'Default',
                                      'missing': 'Missing',
                                      'preparer': 'Preparer',
                                      'validator': 'validator',
                                      'custom_attr': 'Custom attr'})

        self.assertEqual(field.name, 'test')
        self.assertEqual(field.title, 'Title')
        self.assertEqual(field.description, 'Description')
        self.assertEqual(field.readonly, True)
        self.assertEqual(field.default, 'Default')
        self.assertEqual(field.missing, 'Missing')
        self.assertEqual(field.preparer, 'Preparer')
        self.assertEqual(field.validator, 'validator')
        self.assertEqual(field.custom_attr, 'Custom attr')

        self.assertEqual(repr(field), "<Field 'test'>")

    def test_field_bind(self):
        from ptah import form

        orig_field = form.Field(
            'test', **{'title': 'Title',
                       'description': 'Description',
                       'readonly': True,
                       'default': 'Default',
                       'missing': 'Missing',
                       'preparer': 'Preparer',
                       'validator': 'validator',
                       'custom_attr': 'Custom attr'})

        field = orig_field.bind('field.', form.null, {})

        self.assertEqual(field.value, form.null)
        self.assertEqual(field.params, {})
        self.assertEqual(field.name, 'field.test')
        self.assertEqual(field.title, 'Title')
        self.assertEqual(field.description, 'Description')
        self.assertEqual(field.readonly, True)
        self.assertEqual(field.default, 'Default')
        self.assertEqual(field.missing, 'Missing')
        self.assertEqual(field.preparer, 'Preparer')
        self.assertEqual(field.validator, 'validator')
        self.assertEqual(field.custom_attr, 'Custom attr')

        self.assertEqual(repr(field), "<Field 'field.test'>")

    def test_field_field_api(self):
        from ptah import form

        field = form.Field('test')

        self.assertRaises(NotImplementedError, field.serialize, '')
        self.assertRaises(NotImplementedError, field.deserialize, '')

    def test_field_validate(self):
        from ptah import form

        field = form.Field('test')

        self.assertIsNone(field.validate(''))
        self.assertRaises(form.Invalid, field.validate, form.required)

        def validator(field, value):
            raise form.Invalid(field, 'msg')

        field = form.Field('test', validator=validator)
        self.assertRaises(form.Invalid, field.validate, '')

    def test_field_extract(self):
        from ptah import form

        field = form.Field('test')

        widget = field.bind('field.', form.null, {})

        self.assertIs(widget.extract(), form.null)
        self.assertIs(widget.extract(default='test'), 'test')

        widget = field.bind('field.', form.null, {'field.test': 'TEST'})
        self.assertIs(widget.extract(), 'TEST')
        self.assertIs(widget.extract(default='test'), 'TEST')

    def test_field_render(self):
        from ptah import form

        class MyField(form.Field):

            def tmpl_input(self, **args):
                return 'INPUT'

            def tmpl_display(self, **args):
                return 'DISPLAY'

        field = MyField('test')
        widget = field.bind('field.', form.null, {})
        widget.request = {}

        widget.mode = form.FORM_INPUT
        self.assertEqual(widget.render({}), 'INPUT')

        widget.mode = form.FORM_DISPLAY
        self.assertEqual(widget.render({}), 'DISPLAY')

    def test_field_update_mode(self):
        from ptah import form

        request = object()
        field = form.Field('test')
        widget = field.bind('field.', form.null, {})

        widget.update(request)
        self.assertEqual(widget.mode, form.FORM_INPUT)

        field = form.Field('test', readonly=True)
        widget = field.bind('field.', form.null, {})

        widget.update(request)
        self.assertEqual(widget.mode, form.FORM_DISPLAY)

        field = form.Field('test', mode=form.FORM_DISPLAY)
        widget = field.bind('field.', form.null, {})

        widget.update(request)
        self.assertEqual(widget.mode, form.FORM_DISPLAY)

    def test_field_update_value(self):
        from ptah import form

        class MyField(form.Field):
            def serialize(self, value):
                return value
            def deserialize(self, value): # pragma: no cover
                return value

        request = object()

        field = MyField('test')
        widget = field.bind('field.', form.null, {})
        widget.update(request)
        self.assertIs(widget.form_value, None)

        field = MyField('test', default='default value')
        widget = field.bind('field.', form.null, {})
        widget.update(request)
        self.assertIs(widget.form_value, 'default value')

        field = MyField('test', default='default value')
        widget = field.bind('field.', 'content value', {})
        widget.update(request)
        self.assertIs(widget.form_value, 'content value')

        widget = field.bind('field.', form.null, {'field.test': 'form value'})
        widget.update(request)
        self.assertEqual(widget.form_value, 'form value')


class TestFieldFactory(Base):

    def test_field_factory(self):
        from ptah import form

        global MyField
        class MyField(form.Field):
            form.field('my-field')

        self._init_ptah()

        field = form.FieldFactory(
            'my-field', 'test', title='Test field')

        self.assertEqual(field.__field__, 'my-field')

        content = object()
        params = object()

        widget = field.bind('field.', content, params)

        self.assertIsInstance(widget, MyField)
        self.assertIs(widget.value, content)
        self.assertIs(widget.params, params)
        self.assertEqual(widget.name, 'field.test')

    def test_field_faile(self):
        from ptah import form

        field = form.FieldFactory(
            'new-field', 'test', title='Test field')

        self.assertRaises(
            TypeError, field.bind, '', object(), object())



""" base class """
import unittest
import sqlahelper
import sqlalchemy
import transaction
from pyramid import testing
from pyramid.interfaces import IAuthenticationPolicy, IAuthorizationPolicy
from pyramid.authorization import ACLAuthorizationPolicy
from pyramid.authentication import AuthTktAuthenticationPolicy

import ptah
from ptah import config


class Base(unittest.TestCase):

    _settings = {'sqla.url': 'sqlite://',
                 'sqla.cache': False,
                 'auth.secret': 'test'}

    def _makeRequest(self, environ=None): #pragma: no cover
        from pyramid.request import Request
        if environ is None:
            environ = self._makeEnviron()
        return Request(environ)

    def _makeEnviron(self, **extras): #pragma: no cover
        environ = {
            'wsgi.url_scheme':'http',
            'wsgi.version':(1,0),
            'PATH_INFO': '/',
            'SERVER_NAME':'localhost',
            'SERVER_PORT':'8080',
            'REQUEST_METHOD':'GET',
            }
        environ.update(extras)
        return environ

    def _init_ptah(self, handler=None, *args, **kw):
        config.initialize(
            self.p_config, ('ptah', self.__class__.__module__),
            initsettings = False)
        config.initialize_settings(self._settings, self.p_config)

        # create sql tables
        Base = sqlahelper.get_base()
        Base.metadata.drop_all()
        Base.metadata.create_all()
        transaction.commit()

    def _setup_pyramid(self):
        self.request = request = self._makeRequest()
        self.p_config = testing.setUp(request=request)
        self.p_config.get_routes_mapper()
        self.registry = self.p_config.registry
        self.request.registry = self.registry

        policy = AuthTktAuthenticationPolicy(
            'secret', callback= ptah.get_local_roles)

        self.registry.registerUtility(
            policy, IAuthenticationPolicy)

        self.registry.registerUtility(
            ACLAuthorizationPolicy(), IAuthorizationPolicy)

    def _setRequest(self, request): #pragma: no cover
        self.request = request
        self.p_config.end()
        self.p_config.begin(request)

    def setUp(self):
        try:
            engine = sqlahelper.get_engine()
        except: # pragma: no cover
            engine = sqlalchemy.engine_from_config(
                {'sqlalchemy.url': 'sqlite://'})
            sqlahelper.add_engine(engine)

        self._setup_pyramid()

    def tearDown(self):
        config.cleanup_system()
        sm = self.p_config
        sm.__init__('base')
        testing.tearDown()



""" rest api for cms """
from zope import interface
from zope.interface import providedBy
from collections import OrderedDict

import ptah
from ptah import config
from ptah.cms import wrap
from ptah.cms.node import load
from ptah.cms.container import Container

from interfaces import NotFound, CmsException
from interfaces import INode, IBlob, IContent, IContainer
from permissions import View, ModifyContent, DeleteContent


CMS = ptah.RestService('cms', 'Ptah CMS API')


@CMS.action('applications', 'List applications')
def cmsApplications(request, *args):
    apps = []

    for name, factory in ptah.cms.get_app_factories().items():
        root = factory(request)

        try:
            info = wrap(root).info()
        except (AttributeError, CmsException):
            continue

        info['__mount__'] = name
        if name:
            info['__link__'] = '%s/content:%s/%s/'%(
                request.application_url, name, root.__uri__)
        else:
            info['__link__'] = '%s/content/%s/'%(
                request.application_url, root.__uri__)
        apps.append(info)

    return apps


@CMS.action('types', 'List content types')
def cmsTypes(request, *args):
    types = []

    for name, tinfo in ptah.cms.get_types().items():
        types.append((tinfo.title, name, typeInfo(tinfo, request)))

    types.sort()
    return [info for _t, name, info in types]


def typeInfo(tinfo, request):
    info = OrderedDict(
        (('__uri__', tinfo.__uri__),
         ('name', tinfo.name),
         ('title', tinfo.title),
         ('description', tinfo.description),
         ('permission', tinfo.permission),
         ('fieldset', []),
         ))

    fieldset = info['fieldset']

    for field in tinfo.fieldset.fields():
        fieldset.append(
            OrderedDict(
                (('type', field.__field__),
                 ('name', field.name),
                 ('title', field.title),
                 ('description', field.description),
                 ('required', field.required),
                 )))

    return info


@CMS.action('content', 'CMS content')
def cmsContent(request, app='', uri=None, action='', *args):
    name = getattr(request, 'subpath', ('content',))[0]
    if ':' not in name:
        if not action:
            action = uri or ''
        uri = app
        app = u''

    content = None

    appfactory = ptah.cms.get_app_factories().get(app)
    if appfactory is not None:
        root = appfactory(request)
        request.root = root

        if not uri:
            content = root

    if content is None:
        content = load(uri)

    adapters = request.registry.adapters

    action = adapters.lookup(
        (IRestActionClassifier, providedBy(content)),
        IRestAction, name=action, default=None)

    if action:
        if app:
            request.environ['SCRIPT_NAME'] = '%s/content:%s/'%(
                request.environ['SCRIPT_NAME'], app)
        else:
            request.environ['SCRIPT_NAME'] = '%s/content/'%(
                request.environ['SCRIPT_NAME'])

        ptah.check_permission(action.permission, content, request, True)
        res = action.callable(content, request, *args)
        if not res: # pragma: no cover
            res = {}
        return res

    raise NotFound()


class IRestAction(interface.Interface):
    pass


class IRestActionClassifier(interface.Interface):
    pass


class Action(object):
    interface.implements(IRestAction)

    def __init__(self, callable, name, permission):
        self.callable = callable
        self.name = name
        self.title = name
        self.description = callable.__doc__
        self.permission = permission


def restaction(name, context, permission):
    info = config.DirectiveInfo()

    def _register(cfg, callable, name, context, permission):
        ac = Action(callable, name, permission)
        cfg.registry.registerAdapter(
            ac, (IRestActionClassifier, context), IRestAction, name)

    def wrapper(func):
        info.attach(
            config.Action(
                _register,
                (func, name, context, permission),
                discriminator = ('ptah-cms:rest-action', name, context))
            )
        return func

    return wrapper


@restaction('', INode, View)
def nodeInfo(content, request, *args):
    info = wrap(content).info()
    info['__link__'] = '%s%s/'%(request.application_url, content.__uri__)
    return info


@restaction('', IContainer, View)
def containerNodeInfo(content, request, *args):
    """Container information"""
    info = nodeInfo(content, request)

    contents = []
    for item in content.values():
        if not ptah.check_permission(View, item, request): # pragma: no cover
            continue

        contents.append(
            OrderedDict((
                    ('__name__', item.__name__),
                    ('__type__', item.__type_id__),
                    ('__uri__', item.__uri__),
                    ('__container__', isinstance(item, Container)),
                    ('__link__', '%s%s/'%(request.application_url,
                                          item.__uri__)),
                    ('title', item.title),
                    ('description', item.description),
                    ('created', item.created),
                    ('modified', item.modified),
                    )))

    info['__contents__'] = contents
    return info


@restaction('apidoc', INode, ptah.NO_PERMISSION_REQUIRED)
def apidocAction(content, request, *args):
    """api doc"""
    actions = []
    url = request.application_url
    for name, action in request.registry.adapters.lookupAll(
        (IRestActionClassifier, providedBy(content)), IRestAction):

        if not ptah.check_permission(
            action.permission, content, request):
            continue

        actions.append(
            (name, action.title,
             OrderedDict(
                    (('name', name or 'info'),
                     ('link', '%s%s/%s'%(url, content.__uri__, name)),
                     ('title', action.title),
                     ('description', action.description)))))

    actions.sort()
    return [action for _t, _n, action in actions]


@restaction('delete', IContent, DeleteContent)
def deleteAction(content, request, *args):
    """Delete content"""
    content.delete()


@restaction('move', IContent, ModifyContent)
def moveAction(content, request, *args):
    """Move content"""


@restaction('update', IContent, ModifyContent)
def updateAction(content, request, *args):
    """Update content"""
    tinfo = content.__type__

    fieldset = tinfo.fieldset.bind(None, request.POST)
    data, errors = fieldset.extract()
    if errors:
        request.response.status = 500
        return {'errors': errors.msg}

    content.update(**data)
    return nodeInfo(content, request)


@restaction('create', IContainer, View)
def createContentAction(content, request, *args):
    """Create content"""
    name = request.GET.get('name')
    tinfo = request.GET.get('tinfo')

    tinfo = ptah.resolve(tinfo)

    fieldset = tinfo.fieldset.bind(None, request.POST)
    data, errors = fieldset.extract()
    if errors:
        request.response.status = 500
        return {'errors': errors.msg}

    item = wrap(content).create(tinfo.__uri__, name, **data)
    return nodeInfo(item, request)


@restaction('data', IBlob, View)
def blobData(content, request, *args):
    """Download blob"""
    response = request.response

    info = content.info()
    response.content_type = info['mimetype'].encode('utf-8')
    if info['filename']:
        response.headerlist = {
            'Content-Disposition':
            'filename="%s"'%info['filename'].encode('utf-8')}
    response.body = content.read()
    return response



# ptah.cms api

#
from ptah.cms.security import wrap, action
from ptah.cms.interfaces import Error, NotFound, Forbidden

# base content classes
from ptah.cms.node import Node
from ptah.cms.node import load
from ptah.cms.node import load_parents
from ptah.cms.node import get_policy, set_policy

from ptah.cms.content import Content
from ptah.cms.container import Container

# type information
from ptah.cms.tinfo import Type
from ptah.cms.tinfo import TypeInformation
from ptah.cms.tinfo import get_type, get_types

# application root
from ptah.cms.root import get_app_factories
from ptah.cms.root import ApplicationRoot
from ptah.cms.root import ApplicationPolicy
from ptah.cms.root import ApplicationFactory

# blob storage
from ptah.cms.blob import blobStorage
from ptah.cms.interfaces import IBlob
from ptah.cms.interfaces import IBlobStorage

# schemas
from ptah.cms.interfaces import ContentSchema
from ptah.cms.interfaces import ContentNameSchema

# interfaces
from ptah.cms.interfaces import INode
from ptah.cms.interfaces import IContent
from ptah.cms.interfaces import IContainer
from ptah.cms.interfaces import IApplicationRoot
from ptah.cms.interfaces import IApplicationPolicy

# sqlalchemy
from ptah.cms.node import Base, Session

# permissions
from ptah.cms.permissions import View
from ptah.cms.permissions import AddContent
from ptah.cms.permissions import DeleteContent
from ptah.cms.permissions import ModifyContent
from ptah.cms.permissions import ShareContent
from ptah import NOT_ALLOWED
from pyramid.security import ALL_PERMISSIONS
from pyramid.security import NO_PERMISSION_REQUIRED

# events
from ptah.cms.events import ContentEvent
from ptah.cms.events import ContentCreatedEvent
from ptah.cms.events import ContentAddedEvent
from ptah.cms.events import ContentMovedEvent
from ptah.cms.events import ContentModifiedEvent
from ptah.cms.events import ContentDeletingEvent

# cms rest
from ptah.cms.rest import restaction

# content add/edit form helpers
from ptah.cms.forms import AddForm
from ptah.cms.forms import EditForm

# simple ui actions
from ptah.cms.uiactions import uiaction
from ptah.cms.uiactions import list_uiactions



""" uri resolve """
import inspect
from pyramid.httpexceptions import HTTPFound

import ptah
from ptah import form, view, config, manage
from ptah.uri import RESOLVER_ID

from manage import PtahManageRoute


class UriResolver(form.Form):
    view.pview('uri.html', context = PtahManageRoute,
               template = view.template('ptah.manage:templates/uri.pt'))

    fields = form.Fieldset(
        form.LinesField(
            'uri',
            title = 'Uri',
            description = "List of uri's",
            klass = 'xxlarge'))

    uri = None
    rst_to_html = staticmethod(ptah.rst_to_html)

    def form_content(self):
        return {'uri': [self.request.GET.get('uri','')]}

    @form.button('Show', actype=form.AC_PRIMARY)
    def show_handler(self):
        data, errors = self.extract()
        if errors:
            self.message(errors, 'form-error')
        else:
            self.uri = data['uri']

    def update(self):
        super(UriResolver, self).update()

        uri = self.uri
        if uri is None:
            uri = [self.request.GET.get('uri','')]

        resolvers = config.get_cfg_storage(RESOLVER_ID)

        self.data = data = []
        for u in uri:
            if u:
                schema = ptah.extract_uri_schema(u)
                resolver = resolvers.get(schema)
                info = {'uri': u,
                        'resolver': None,
                        'module': None,
                        'line': None,
                        'obj': None,
                        'cls': None,
                        'clsdoc': None}

                if resolver is not None:
                    info['resolver'] = resolver.__name__
                    info['r_doc'] = ptah.rst_to_html(resolver.__doc__ or u'')
                    info['module'] = resolver.__module__
                    info['name'] = '%s.%s'%(
                        resolver.__module__, resolver.__name__)
                    info['line'] = inspect.getsourcelines(resolver)[-1]

                    obj = ptah.resolve(u)
                    info['obj'] = obj

                    if obj is not None:
                        cls = getattr(obj, '__class__', None)
                        info['cls'] = cls
                        info['clsdoc'] = ptah.rst_to_html(
                            getattr(cls, '__doc__', u'') or u'')

                        if cls is not None:
                            info['clsmod'] = cls.__module__
                            info['clsline'] = inspect.getsourcelines(cls)[-1]

                data.append(info)


class UriIntrospection(object):
    """ uri introspection view """

    title = 'Uri resolver'
    manage.introspection('ptah:uri-resolver')

    tmpl = view.template('ptah.manage:templates/directive-uriresolver.pt')

    def __init__(self, request):
        self.request = request

    def renderActions(self, *actions):
        return self.tmpl(
            actions = actions,
            rst_to_html = ptah.rst_to_html,
            manage_url = manage.CONFIG.manage_url,
            request = self.request)



""" settings module """
from ptah import view, manage
from ptah.config.settings import get_settings


class SettingsModule(manage.PtahModule):
    __doc__ = u'The current settings which include defaults not used by .ini'

    title = 'Settings'
    manage.module('settings')


class SettingsView(view.View):
    view.pview(
        context = SettingsModule,
        template = view.template('ptah.manage:templates/settings.pt'))

    __doc__ = "Settings module page."
    __intr_path__ = '/ptah-manage/settings/index.html'

    def update(self):
        groups = get_settings().items()
        groups.sort()

        data = []
        for name, group in groups:
            title = group.title or name
            description = group.description

            schema = []
            for node in group.schema:
                value = node.serialize(group[node.name])

                schema.append(
                    ({'name': '%s.%s'%(name, node.name),
                      'value': '%s: %s'%(node.typ.__class__.__name__, value),
                      'title': node.title,
                      'description': node.description,
                      'default': node.default}))

            data.append(
                ({'title': title,
                  'description': description,
                  'schema': schema}))

        return {'data': data}



""" snippet implementation """
import logging
from zope import interface
from pyramid.httpexceptions import HTTPNotFound

from ptah import config
from ptah.view.base import View
from ptah.view.customize import LayerWrapper
from ptah.view.interfaces import ISnippet

log = logging.getLogger('ptah.view')

STYPE_ID = 'ptah.view:snippettype'


class Snippet(View):
    interface.implements(ISnippet)

    template = None
    _params = None

    def render(self):
        kwargs = self._params or {}
        kwargs.update({'view': self,
                       'context': self.context,
                       'request': self.request})

        return self.template(**kwargs)

    def __call__(self, *args, **kw):
        self._params = self.update()
        return self.render()


class SnippetType(object):

    def __init__(self, name, context, title, description):
        self.name = name
        self.context = context
        self.title = title
        self.description = description


def render_snippet(stype, context, request):
    snippet = request.registry.queryMultiAdapter(
        (context, request), ISnippet, stype)
    if snippet is None:
        raise HTTPNotFound

    try:
        return snippet()
    except Exception, e:
        log.exception(str(e))
        raise


def snippettype(name, context=None, title='', description=''):
    stype = SnippetType(name, context, title, description)

    info = config.DirectiveInfo()
    info.attach(
        config.Action(
            lambda config, name, stype: \
                config.get_cfg_storage(STYPE_ID).update({name: stype}),
            (name, stype,),
            discriminator = (STYPE_ID, name), order = 1))


def register_snippet(name, context=None, klass=None, template=None, layer=''):
    info = config.DirectiveInfo()

    discriminator = ('ptah.view:snippet', name, context, layer)

    info.attach(
        config.Action(
            LayerWrapper(register_snippet_impl, discriminator),
            (klass, name, context, template),
            discriminator = discriminator)
        )


def register_snippet_impl(cfg, klass, stype, context, template):
    cdict = {}
    if template is not None:
        cdict['template'] = template

    # find SnippetType info
    if stype not in cfg.get_cfg_storage(STYPE_ID):
        log.warning("Can't find SnippetType %s", stype)

    requires = [context, interface.Interface]

    # Build a new class
    if klass is not None and issubclass(klass, Snippet):
        snippet_class = klass
        for attr, value in cdict.items():
            setattr(snippet_class, attr, value)
    else:
        # Build a new class
        if klass is None:
            bases = (Snippet,)
        else:
            bases = (klass, Snippet)

        snippet_class = type('Snippet %s'%klass, bases, cdict)

    # register snippet
    cfg.registry.registerAdapter(
        snippet_class, requires, ISnippet, name = stype)



""" paste commands """
import os.path, shutil, textwrap
from pyramid import testing
from paste.script.command import Command

from ptah import config
from ptah.view import resources


grpTitleWrap = textwrap.TextWrapper(
    initial_indent='* ',
    subsequent_indent='  ',
    width = 80)

nameWrap = textwrap.TextWrapper(
    initial_indent='  - ',
    subsequent_indent='    ', width = 80)

overWrap = textwrap.TextWrapper(
    initial_indent='      ',
    subsequent_indent='      ', width = 80)

nameTitleWrap = textwrap.TextWrapper(
    initial_indent='       ',
    subsequent_indent='       ', width = 80)


class StaticCommand(Command):
    """ 'static' paste command"""

    summary = "ptah static assets management"
    usage = ""
    group_name = "ptah"
    parser = Command.standard_parser(verbose=False)
    parser.add_option('-l', '--list', dest='section',
                      action="store_true",
                      help = 'List registered static sections')
    parser.add_option('-d', '--dump', dest='dump',
                      help = 'Dump static assets')

    _include = None

    def command(self):
        # load all ptah packages
        pconfig = testing.setUp()
        config.initialize(pconfig, self._include, autoinclude=True)
        registry = config.get_cfg_storage(resources.STATIC_ID)

        if self.options.dump:
            basepath = self.options.dump.strip()
            if not os.path.exists(basepath):
                os.makedirs(basepath)

            if not os.path.isdir(basepath):
                print "Output path is not directory."
                return

            items = registry.items()
            items.sort()
            for name, (path, pkg) in items:
                print "* Coping from '%s' %s"%(pkg, path)

                d = resources.buildTree(path)
                di = d.items()
                di.sort()

                for p, _t in di:
                    bp = os.path.join(basepath, name, p)
                    dest_dir = os.path.split(bp)[0]
                    if not os.path.exists(dest_dir):
                        os.makedirs(dest_dir)

                    forp = '%s/%s'%(pkg, p.split(pkg, 1)[-1])
                    if os.path.exists(bp):
                        print '   skipping ../%s'%forp
                    else:
                        print '   coping ../%s'%forp
                        shutil.copyfile(os.path.join(path, p), bp)

                print

            print basepath
            return

        # list static sections
        if self.options.section:
            items = registry.items()
            items.sort()

            for name, (path, pkg) in items:
                print grpTitleWrap.fill(name)
                print nameWrap.fill('by: %s'%pkg)
                p = path.split(pkg, 1)[-1]
                print nameTitleWrap.fill(' ../%s%s'%(pkg, p))
                print



""" Base content class """
import sqlalchemy as sqla
from sqlalchemy.ext.hybrid import hybrid_property

from zope import interface
from ptah import config, form

import ptah
from ptah.cms import events
from ptah.cms.node import Node, Session
from ptah.cms.interfaces import Error, IContent
from ptah.cms.security import action
from ptah.cms.permissions import DeleteContent, ModifyContent


class Content(Node):
    """ Base class for content objects. A content class should inherit from
    `Content` to participate in content hierarchy traversal.

    .. attribute:: __path__

       A string used by the :py:class:`ptah.cms.ContentTraverser` which is
       used for efficient resolution of URL structure to content models.
       This is internal implementation and manually editing it can break
       your hierarchy.

    .. attribute:: __name__

       This is the identifier in a container if you are using containment and
       hierarchies.

    .. attribute:: title

       Content title which is editable by end user.

    .. attribute:: description

       Content description which is editable by end user.

    .. attribute:: view

       A URI which can be resolved with :py:func:`ptah.resolve` function
       which represents the 'default' view for content. Akin to index.html
       or default.php in Apache.

    .. attribute:: created

       Content creation time which is set by
       :py:func:`ptah.cms.events.createdHandler` during object creation.

       :type: :py:class:`datetime.datetime`

    .. attribute:: modified

       Content modification time which is set by
       :py:func:`ptah.cms.events.modifiedHandler` during object modification.

       :type: :py:class:`datetime.datetime`

    .. attribute:: effective

       :type: :py:class:`datetime.datetime` or None

    .. attribute:: expires

       :type: :py:class:`datetime.datetime` or None

    .. attribute:: creators

       a :py:class:`ptah.JsonListType` which contains sequence of users.  Using
       principal URIs is a good idea.

    .. attribute:: subjects

       a :py:class:`ptah.JsonListType` which contains sequence of subjects.
       Holding a sequence of URIs could resolve to subject objects. Or you can
       use strings.

    .. attribute: publisher

       a Unicode string which should identify the publisher.

    .. attribute: contributors

       a :py:class:`ptah.JsonListType` which contains sequence of contributors.
       You could keep a sequence of principal URIs.
    """

    interface.implements(IContent)

    __tablename__ = 'ptah_content'

    __id__ = sqla.Column('id', sqla.Integer,
                         sqla.ForeignKey('ptah_nodes.id'), primary_key=True)
    __path__ = sqla.Column('path', sqla.Unicode, default=u'')
    __name_id__ = sqla.Column('name', sqla.Unicode(255))

    title = sqla.Column(sqla.Unicode, default=u'')
    description = sqla.Column(sqla.Unicode, default=u'',
                              info = {'missing': u'', 'field_type': 'textarea'})
    view = sqla.Column(sqla.Unicode, default=u'')

    created = sqla.Column(sqla.DateTime)
    modified = sqla.Column(sqla.DateTime)
    effective = sqla.Column(sqla.DateTime)
    expires = sqla.Column(sqla.DateTime)

    creators = sqla.Column(ptah.JsonListType(), default=[])
    subjects = sqla.Column(ptah.JsonListType(), default=[])
    publisher = sqla.Column(sqla.Unicode, default=u'')
    contributors = sqla.Column(ptah.JsonListType(), default=[])

    # sql queries
    _sql_get = ptah.QueryFreezer(
        lambda: Session.query(Content)
        .filter(Content.__uri__ == sqla.sql.bindparam('uri')))

    _sql_get_in_parent = ptah.QueryFreezer(
        lambda: Session.query(Content)
            .filter(Content.__name_id__ == sqla.sql.bindparam('key'))
            .filter(Content.__parent_uri__ == sqla.sql.bindparam('parent')))

    _sql_parent = ptah.QueryFreezer(
        lambda: Session.query(Content)
            .filter(Content.__uri__ == sqla.sql.bindparam('parent')))

    def __init__(self, **kw):
        super(Content, self).__init__(**kw)

        if self.__name__ and self.__parent__ is not None:
            self.__path__ = '%s%s/'%(self.__parent__.__path__, self.__name__)

    @hybrid_property
    def __name__(self):
        return self.__name_id__

    @__name__.setter
    def __name__(self, value):
        self.__name_id__ = value

    def __resource_url__(self, request, info):
        return '%s%s'%(request.root.__root_path__,
                       self.__path__[len(request.root.__path__):])

    @action(permission=DeleteContent)
    def delete(self):
        parent = self.__parent__
        if parent is None:
            parent = self.__parent_ref__

        if parent is None:
            raise Error("Can't find parent")

        del parent[self]

    @action(permission=ModifyContent)
    def update(self, **data):
        if self.__type__:
            tinfo = self.__type__

            for field in tinfo.fieldset.fields():
                val = data.get(field.name, field.default)
                if val is not form.null:
                    setattr(self, field.name, val)

            config.notify(events.ContentModifiedEvent(self))

    def _extra_info(self, info):
        if self.__type__:
            fieldset = self.__type__.fieldset.bind()
            for field in fieldset.fields():
                val = getattr(self, field.name, field.default)
                info[field.name] = field.serialize(val)

        info['view'] = self.view
        info['created'] = self.created
        info['modified'] = self.modified
        info['effective'] = self.effective
        info['expires'] = self.expires

    def info(self):
        info = super(Content, self).info()
        info['__name__'] = self.__name__
        info['__content__'] = True
        info['__container__'] = False
        self._extra_info(info)
        return info



import inspect
from ptah import config

import ptah
from permissions import View
from interfaces import NotFound, Forbidden


def wrap(content):
    if isinstance(content, basestring):
        content = ptah.cms.load(content)
    else:
        ptah.cms.load_parents(content)

    if content is None:
        raise NotFound()

    return NodeWrapper(content)


class NodeWrapper(object):

    def __init__(self, content):
        self._content = content
        self._actions = getattr(content, '__ptahcms_actions__', None)

    def __getattr__(self, action):
        if not self._actions or action not in self._actions:
            raise NotFound(action)

        fname, permission = self._actions[action]
        if permission:
            if not ptah.check_permission(permission, self._content):
                raise Forbidden(action)

        return ActionWrapper(self._content, fname)


class ActionWrapper(object):

    def __init__(self, content, action):
        self.content = content
        self.action = action

    def __call__(self, *args, **kw):
        return getattr(self.content, self.action)(*args, **kw)


def action(func=None, name=None, permission = View):
    info = config.DirectiveInfo(allowed_scope=('class',))

    actions = info.locals.get('__ptahcms_actions__', None)
    if actions is None:
        actions = {}
        info.locals['__ptahcms_actions__'] = actions

    def wrapper(func, name=name):
        if name is None:
            name = func.__name__

        actions[name] = (func.__name__, permission)

        return func

    if func is not None:
        return wrapper(func)
    return wrapper


def build_class_actions(cls):
    actions = cls.__dict__.get('__ptahcms_actions__', None)
    if actions is None:
        actions = {}

    # get actions from parents
    mro = inspect.getmro(cls)

    for idx in range(len(mro)-1, 0, -1):
        a = getattr(mro[idx], '__ptahcms_actions__', None)
        if a is not None:
            for name, action in a.items():
                if name not in actions:
                    actions[name] = action

    cls.__ptahcms_actions__ = actions
